@@ -8,11 +8,12 @@
 
 import inspect
 import re
-import ctypes
 import ctypes.util
 from . import help as help_module
 from . import ui_utils
 from . import log
+from . import constants
+from .utils import fuzzy_get
 
 
 logger = log.setup_logger(__name__)
@@ -230,44 +231,10 @@ def _cd_completer(args, context):
 
 
 def _help_completer(args, context):
-    'TODO: make better completion'
-    return help_module.list_help_topics() + context.current_level().get_completions()
-
-
-def fuzzy_get(items, s):
-    """
-    Finds s in items using a fuzzy
-    matching algorithm:
-
-    1. if exact match, return value
-    2. if unique prefix, return value
-    3. if unique prefix substring, return value
-    """
-    found = items.get(s)
-    if found:
-        return found
-
-    def fuzzy_match(rx):
-        try:
-            matcher = re.compile(rx, re.I)
-            matches = [c
-                       for m, c in items.items()
-                       if matcher.match(m)]
-            if len(matches) == 1:
-                return matches[0]
-        except re.error as e:
-            raise ValueError(e)
-        return None
-
-    # prefix match
-    m = fuzzy_match(s + '.*')
-    if m:
-        return m
-    # substring match
-    m = fuzzy_match('.*'.join(s) + '.*')
-    if m:
-        return m
-    return None
+    current_level_completions = context.current_level().get_completions()
+    if context.current_level().name == 'root':
+        return help_module.list_help_topics() + current_level_completions
+    return current_level_completions
 
 
 class UI(object):
@@ -415,17 +382,22 @@ Examples:
 ....
 ''')
     @completer(_help_completer)
-    def do_help(self, context, subject=None, subtopic=None):
+    def do_help(self, context, *args):
         """usage: help topic|level|command"""
-        h = help_module.help_contextual(context.level_name(), subject, subtopic)
-        h.paginate()
+        levels = [ui.name for ui in context.stack]
+        levels = levels[1:]     # drop the 1st element "root"
+        levels.extend(args)
+        h = help_module.help_contextual(levels)
+        if h:
+            h.paginate()
         context.command_name = ""
 
     def get_completions(self):
         '''
         return tab completions
         '''
-        return [x for x in self._children.keys() if x not in self._aliases]
+        return [x for x in self._children.keys() if
+                x not in self._aliases]
 
     def get_child(self, child):
         '''
@@ -454,7 +426,7 @@ Examples:
         return cls._children
 
     @classmethod
-    def init_ui(cls):
+    def init_ui(cls, path: list[str]):
         def get_if_command(attr):
             "Return the named attribute if it's a command"
             child = getattr(cls, attr)
@@ -466,10 +438,11 @@ Examples:
                 aliases.append(alias)
                 children[alias] = info
 
-        def add_help(info):
+        def add_help(path: list[str], info):
             "Add static help to the help system"
             if info.short_help:
                 entry = help_module.HelpEntry(info.short_help, info.long_help, generated=True)
+                help_module.add_help(path + [info.name], entry)
             elif info.type == 'command':
                 entry = help_module.HelpEntry(
                     'Help for command ' + info.name,
@@ -477,23 +450,12 @@ Examples:
                     'Usage: %s %s' % (info.name,
                                       ui_utils.pretty_arguments(info.function, nskip=2)),
                     generated=True)
+                help_module.add_help(path + [info.name], entry)
             elif info.type == 'level':
                 entry = help_module.HelpEntry('Help for level ' + info.name,
                                               'Note: This level is not documented.\n',
                                               generated=True)
-            if info.type == 'command':
-                help_module.add_help(entry, level=cls.name, command=info.name)
-            elif info.type == 'level':
-                help_module.add_help(entry, level=info.name)
-
-        def prepare(children, child, aliases):
-            info = ChildInfo(child, cls)
-            if info.type == 'command' and not is_valid_command_function(info.function):
-                raise ValueError("Invalid command function: %s.%s" %
-                                 (cls.__name__, info.function.__name__))
-            children[info.name] = info
-            add_aliases(children, info, aliases)
-            add_help(info)
+                help_module.add_help(path + [info.name], entry)
 
         children = {}
         aliases = []
@@ -502,9 +464,17 @@ Examples:
                 continue
             child = get_if_command(child_name)
             if child:
-                prepare(children, child, aliases)
-        setattr(cls, '_children', children)
-        setattr(cls, '_aliases', aliases)
+                info = ChildInfo(child, cls)
+                if info.type == 'command' and not is_valid_command_function(info.function):
+                    raise ValueError("Invalid command function: %s.%s" %
+                                     (cls.__name__, info.function.__name__))
+                elif info.type == 'level' and info.level:
+                    info.children = info.level.init_ui(path + [info.name])
+                children[info.name] = info
+                add_aliases(children, info, aliases)
+                add_help(path, info)
+        cls._children = children
+        cls._aliases = aliases
         return children
 
 
@@ -556,8 +526,6 @@ class ChildInfo(object):
         self.completer = maybe('_completer', None)
         self.parent = parent
         self.children = {}
-        if self.type == 'level' and self.level:
-            self.children = self.level.init_ui()
 
     def complete(self, context, args):
         '''
@@ -568,15 +536,13 @@ class ChildInfo(object):
         '''
         ret = []
         if self.completer is not None:
-            if sort_completion_inst is not None and sort_completion_inst.value == 0:
-                # Restore the original value of rl_sort_completion_matches
-                # before calling the completer again
-                sort_completion_inst.value = orig_sort_completion_value
+            disable_custom_sort_order()
             specs = inspect.getfullargspec(self.completer)
             if 'context' in specs.args:
                 ret = self.completer([self.name] + args, context)
             else:
                 ret = self.completer([self.name] + args)
+            ret = ret if already_used_custom_sort_order() else sorted(ret)
         return ret
 
     def __repr__(self):
@@ -599,6 +565,22 @@ def _check_args(fn, expected):
                          ": Expected method with signature " + repr(expected))
 
 
+def enable_custom_sort_order():
+    if sort_completion_inst is not None:
+        sort_completion_inst.value = 0
+
+
+def already_used_custom_sort_order():
+    return sort_completion_inst is not None and sort_completion_inst.value == 0
+
+
+def disable_custom_sort_order():
+    if already_used_custom_sort_order():
+        # Restore the original value of rl_sort_completion_matches
+        # before calling the completer again
+        sort_completion_inst.value = orig_sort_completion_value
+
+
 readline_path = ctypes.util.find_library('readline')
 sort_completion_inst = None
 if readline_path:
