@@ -7,17 +7,19 @@ from datetime import datetime
 from contextlib import contextmanager
 from crmsh import utils as crmshutils
 from . import config
+from crmsh import ra
 from crmsh import log
 from crmsh.sh import ShellUtils
+from crmsh import constants
 
 
 logger = log.setup_logger(__name__)
 
 
-CRED = '\033[31m'
-CYELLOW = '\033[33m'
-CGREEN = '\033[32m'
-CEND = '\033[0m'
+CRED = constants.RED
+CYELLOW = constants.YELLOW
+CGREEN = constants.GREEN
+CEND = constants.END
 
 LEVEL = {
     "info": logging.INFO,
@@ -117,8 +119,9 @@ class FenceInfo(object):
     @property
     def fence_action(self):
         action_result = crmshutils.get_property("stonith-action")
-        if action_result is None or action_result not in ["off", "poweroff", "reboot"]:
-            msg_error("Cluster property \"stonith-action\" should be reboot|off|poweroff")
+        supported_actions = ra.get_property_options("stonith-action")
+        if action_result is None or action_result not in supported_actions:
+            msg_error(f"Cluster property \"stonith-action\" should be {'|'.join(supported_actions)}")
             return None
         return action_result
 
