@@ -6,7 +6,9 @@ import subprocess
 import sys
 import typing
 
+import crmsh.constants
 import crmsh.parallax
+import crmsh.user_of_host
 import crmsh.utils
 
 
@@ -120,8 +122,8 @@ class PasswordlessHaclusterAuthenticationFeature(Feature):
     def check_quick(self) -> bool:
         for key_type in self.KEY_TYPES:
             try:
-                os.stat('{}/{}'.format(self.SSH_DIR, key_type))
-                os.stat('{}/{}.pub'.format(self.SSH_DIR, key_type))
+                os.stat('{}/id_{}'.format(self.SSH_DIR, key_type))
+                os.stat('{}/id_{}.pub'.format(self.SSH_DIR, key_type))
                 return True
             except FileNotFoundError:
                 pass
@@ -131,7 +133,7 @@ class PasswordlessHaclusterAuthenticationFeature(Feature):
         try:
             for node in nodes:
                 subprocess.check_call(
-                    ['sudo', 'su', '-', 'hacluster', '-c', 'ssh hacluster@{} true'.format(node)],
+                    ['sudo', 'su', '-', 'hacluster', '-c', 'ssh {} hacluster@{} true'.format(crmsh.constants.SSH_OPTION, node)],
                     stdin=subprocess.DEVNULL,
                     stdout=subprocess.DEVNULL,
                     stderr=subprocess.DEVNULL,
@@ -145,7 +147,13 @@ class PasswordlessHaclusterAuthenticationFeature(Feature):
         logger.debug("setup passwordless ssh authentication for user hacluster")
         local_node = crmsh.utils.this_node()
         remote_nodes = set(nodes)
-        remote_nodes.remove(local_node)
+        try:
+            remote_nodes.remove(local_node)
+        except KeyError:
+            # bsc#1223438: local nodename not in cib
+            # init_ssh_impl should work even if the local node is included in user_node_list
+            # although this is not a designed usage
+            logger.warning("local node %s is not found in cluster node list %s", local_node, remote_nodes)
         remote_nodes = list(remote_nodes)
         crmsh.parallax.parallax_run(
             nodes,
@@ -157,6 +165,35 @@ class PasswordlessHaclusterAuthenticationFeature(Feature):
             crmsh.bootstrap.change_user_shell('hacluster', node)
 
 
+class PasswordlessPrimaryUserAuthenticationFeature(Feature):
+    def check_quick(self) -> bool:
+        local_node = crmsh.utils.this_node()
+        try:
+            crmsh.utils.user_of(local_node)
+            return True
+        except crmsh.user_of_host.UserNotFoundError:
+            return False
+
+    def check_local(self, nodes: typing.Iterable[str]) -> bool:
+        try:
+            for node in nodes:
+                crmsh.utils.user_of(node)
+        except crmsh.user_of_host.UserNotFoundError:
+            return False
+        try:
+            crmsh.parallax.parallax_call(nodes, 'true')
+            return True
+        except ValueError:
+            return False
+
+    def fix_local(self, nodes: typing.Iterable[str], ask: typing.Callable[[str], None]) -> None:
+        logger.warning('Passwordless ssh is not initialized. Use `crm cluster init ssh` (on one node) and `crm cluster join ssh -c <init-node>` (on other nodes) to set it up.')
+        raise FixFailure
+
+    def fix_cluster(self, nodes: typing.Iterable[str], ask: typing.Callable[[str], None]) -> None:
+        return self.fix_local(nodes, ask)
+
+
 def main_check_local(args) -> int:
     try:
         feature = Feature.get_feature_by_name(args.feature)()
