@@ -216,7 +216,7 @@ LOGGING_CFG = {
         },
         "file": {
             "format": "%(asctime)s {} %(name)s: %(levelname)s: %(message)s".format(socket.gethostname()),
-            "datefmt": "%b %d %H:%M:%S",
+            "datefmt": "%Y-%m-%dT%H:%M:%S%z",
         }
     },
     "filters": {
@@ -402,8 +402,10 @@ class LoggerUtils(object):
         To wait and mark something finished, start with BEGIN msg, end of END msg
         """
         self.logger.info("BEGIN %s", msg)
+        pb = ProgressBar()
         try:
-            yield ProgressBar()
+            yield pb
+            pb._end()
         except Exception:
             self.logger.error("FAIL %s", msg)
             raise
@@ -431,8 +433,9 @@ class LoggerUtils(object):
         """
         while True:
             ans = self.wait_input("{} (y/n)? ".format(msg.strip("? ")))
-            if ans:
-                return ans.lower() == "y"
+            if not ans or ans.lower() not in ('y', 'n'):
+                continue
+            return ans.lower() == 'y'
 
     def syntax_err(self, s, token='', context='', msg=''):
         err = "syntax"
@@ -489,7 +492,7 @@ class LoggerUtils(object):
     def update_err(self, obj_id, cibadm_opt, xml, rc):
         CIB_PERMISSION_DENIED_CODE = 54
         task_table = {"-U": "update", "-D": "delete", "-P": "patch"}
-        task = task_table.get(cibadmin_opt, "replace")
+        task = task_table.get(cibadm_opt, "replace")
         self.logger.error("could not %s %s (rc=%d)", task, obj_id, int(rc))
         if int(rc) == CIB_PERMISSION_DENIED_CODE:
             self.logger.info("Permission denied.")
@@ -514,6 +517,24 @@ class ProgressBar:
         sys.stdout.write(line)
         sys.stdout.flush()
 
+    def _end(self):
+        try:
+            width, _ = os.get_terminal_size()
+        except OSError:
+            # not a terminal
+            return
+        if self._i == 0:
+            pass
+        elif self._i < width:
+            line = '\r{}\n'.format('.' * self._i)
+            sys.stdout.write(line)
+        else:
+            # the terminal is resized and narrower than the progress bar printed before
+            # just write an LF in this case
+            sys.stdout.write('\n')
+        sys.stdout.flush()
+
+
 
 def setup_logging(only_help=False):
     """
