@@ -2,13 +2,11 @@ import os
 import re
 import socket
 import functools
-import subprocess
-import tempfile
 import typing
 from enum import Enum
 
 import crmsh.parallax
-from . import constants, sh
+from . import sh
 from . import utils
 from . import parallax
 from . import corosync
@@ -17,6 +15,8 @@ from . import bootstrap
 from . import lock
 from . import log
 from .service_manager import ServiceManager
+from .sbd import SBDManager, SBDTimeout, SBDUtils
+
 
 logger = log.setup_logger(__name__)
 logger_utils = log.LoggerUtils(logger)
@@ -52,13 +52,13 @@ def evaluate_qdevice_quorum_effect(mode, diskless_sbd=False, is_stage=False):
     elif mode == QDEVICE_ADD and not is_stage:
         # Add qdevice from init process, safe to restart
         return QdevicePolicy.QDEVICE_RESTART
-    elif xmlutil.CrmMonXmlParser().is_any_resource_running():
-        # will lose quorum, and with RA running
+    elif xmlutil.CrmMonXmlParser().is_non_stonith_resource_running():
+        # will lose quorum, with non-stonith resource running
         # no reload, no restart cluster service
         # just leave a warning
         return QdevicePolicy.QDEVICE_RESTART_LATER
     else:
-        # will lose quorum, without RA running
+        # will lose quorum, without resource running or just stonith resource running
         # safe to restart cluster service
         return QdevicePolicy.QDEVICE_RESTART
 
@@ -514,38 +514,42 @@ class QDevice(object):
         self.fetch_p12_from_cluster()
         self.import_p12_on_local()
 
-    def write_qdevice_config(self):
+    def write_qdevice_config(self) -> None:
         """
         Write qdevice attributes to config file
         """
-        p = corosync.Parser(utils.read_from_file(corosync.conf()))
-        p.remove("quorum.device")
-        p.add('quorum', corosync.make_section('quorum.device', []))
-        p.set('quorum.device.votes', '1')
-        p.set('quorum.device.model', 'net')
-        p.add('quorum.device', corosync.make_section('quorum.device.net', []))
-        p.set('quorum.device.net.tls', self.tls)
-        p.set('quorum.device.net.host', self.qnetd_addr)
-        p.set('quorum.device.net.port', self.port)
-        p.set('quorum.device.net.algorithm', self.algo)
-        p.set('quorum.device.net.tie_breaker', self.tie_breaker)
+        inst = corosync.ConfParser()
+        qdevice_config_dict = {
+                "model": "net",
+                "net": {
+                    "tls": self.tls,
+                    "host": self.qnetd_addr,
+                    "port": self.port,
+                    "algorithm": self.algo,
+                    "tie_breaker": self.tie_breaker
+                    }
+                }
+        if self.algo == "ffsplit":
+            # According to man corosync-qdevice, if the algorithm is lms, do not set votes
+            qdevice_config_dict['votes'] = 1
+        inst.set("quorum.device", qdevice_config_dict)
         if self.cmds:
-            p.add('quorum.device', corosync.make_section('quorum.device.heuristics', []))
-            p.set('quorum.device.heuristics.mode', self.mode)
+            heuristics_dict = {"mode": self.mode}
             for i, cmd in enumerate(self.cmds.strip(';').split(';')):
                 cmd_name = re.sub("[.-]", "_", os.path.basename(cmd.split()[0]))
                 exec_name = "exec_{}{}".format(cmd_name, i)
-                p.set('quorum.device.heuristics.{}'.format(exec_name), cmd)
-        utils.str2file(p.to_string(), corosync.conf())
+                heuristics_dict[exec_name] = cmd
+            inst.set("quorum.device.heuristics", heuristics_dict)
+        inst.save()
 
     @staticmethod
     def remove_qdevice_config():
         """
         Remove configuration of qdevice
         """
-        p = corosync.Parser(utils.read_from_file(corosync.conf()))
-        p.remove("quorum.device")
-        utils.str2file(p.to_string(), corosync.conf())
+        inst = corosync.ConfParser()
+        inst.remove("quorum.device")
+        inst.save()
 
     @staticmethod
     def remove_qdevice_db(addr_list=[]):
@@ -560,11 +564,11 @@ class QDevice(object):
         parallax.parallax_call(node_list, cmd)
 
     @classmethod
-    def remove_certification_files_on_qnetd(cls):
+    def remove_certification_files_on_qnetd(cls) -> None:
         """
         Remove this cluster related .crq and .crt files on qnetd
         """
-        if not utils.is_qdevice_configured():
+        if not corosync.is_qdevice_configured():
             return
         qnetd_host = corosync.get_value('quorum.device.net.host')
         cluster_name = corosync.get_value('totem.cluster_name')
@@ -575,17 +579,33 @@ class QDevice(object):
         cmd = "test -f {crq_file} && rm -f {crq_file}".format(crq_file=cls_inst.qdevice_crq_on_qnetd)
         shell.get_stdout_or_raise_error(cmd, qnetd_host)
 
-    def config_qdevice(self):
+    def config_qdevice(self) -> None:
         """
         Update configuration and reload corosync if necessary
         """
         self.write_qdevice_config()
-        if not corosync.is_unicast():
-            corosync.add_nodelist_from_cmaptool()
         with logger_utils.status_long("Update configuration"):
-            bootstrap.update_expected_votes()
+            corosync.configure_two_node(qdevice_adding=True)
+            bootstrap.sync_file(corosync.conf())
             if self.qdevice_reload_policy == QdevicePolicy.QDEVICE_RELOAD:
-                utils.cluster_run_cmd("crm corosync reload")
+                sh.cluster_shell().get_stdout_or_raise_error("corosync-cfgtool -R")
+
+    def config_qnetd_port(self):
+        """
+        Enable qnetd port in firewalld
+        """
+        if not ServiceManager().service_is_active("firewalld.service", self.qnetd_addr):
+            return
+        if utils.check_port_open(self.qnetd_addr, self.port):
+            return
+        shell = sh.cluster_shell()
+        cmd = f"firewall-cmd --add-port={self.port}/tcp --permanent"
+        rc, out, err = shell.get_rc_stdout_stderr_without_input(self.qnetd_addr, cmd)
+        if rc != 0 and err:
+            logger.error("Failed to add port {} to firewalld on {}: {}".format(self.port, self.qnetd_addr, err))
+            return
+        logger.info("Add port {} to firewalld on {}".format(self.port, self.qnetd_addr))
+        shell.get_stdout_or_raise_error("firewall-cmd --reload", self.qnetd_addr)
 
     def start_qdevice_service(self):
         """
@@ -610,15 +630,13 @@ class QDevice(object):
         """
         Adjust SBD_WATCHDOG_TIMEOUT when configuring qdevice and diskless SBD
         """
-        from .sbd import SBDManager, SBDTimeout
-        utils.check_all_nodes_reachable()
-        self.using_diskless_sbd = SBDManager.is_using_diskless_sbd()
+        self.using_diskless_sbd = SBDUtils.is_using_diskless_sbd()
         # add qdevice after diskless sbd started
         if self.using_diskless_sbd:
-            res = SBDManager.get_sbd_value_from_config("SBD_WATCHDOG_TIMEOUT")
+            res = SBDUtils.get_sbd_value_from_config("SBD_WATCHDOG_TIMEOUT")
             if not res or int(res) < SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE:
                 sbd_watchdog_timeout_qdevice = SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE
-                SBDManager.update_configuration({"SBD_WATCHDOG_TIMEOUT": str(sbd_watchdog_timeout_qdevice)})
+                SBDManager.update_sbd_configuration({"SBD_WATCHDOG_TIMEOUT": str(sbd_watchdog_timeout_qdevice)})
                 utils.set_property("stonith-timeout", SBDTimeout.get_stonith_timeout())
 
     @qnetd_lock_for_same_cluster_name
@@ -633,6 +651,7 @@ class QDevice(object):
         self.adjust_sbd_watchdog_timeout_with_qdevice()
         self.qdevice_reload_policy = evaluate_qdevice_quorum_effect(QDEVICE_ADD, self.using_diskless_sbd, self.is_stage)
         self.config_qdevice()
+        self.config_qnetd_port()
         self.start_qdevice_service()
 
     @staticmethod
