@@ -10,7 +10,7 @@ import stat
 import pwd
 import datetime
 from subprocess import TimeoutExpired
-from typing import List
+from typing import List, Optional
 
 import crmsh.user_of_host
 from crmsh import log, sh, corosync
@@ -23,11 +23,11 @@ from crmsh.service_manager import ServiceManager
 logger = log.setup_report_logger(__name__)
 
 
-def get_corosync_log() -> str:
+def get_corosync_log() -> Optional[str]:
     """
     Get the path of the corosync log file
     """
-    corosync_log = ""
+    corosync_log = None
     corosync_conf_path = corosync.conf()
     if os.path.exists(corosync_conf_path):
         corosync_log = corosync.get_value("logging.logfile")
@@ -36,7 +36,7 @@ def get_corosync_log() -> str:
     return corosync_log
 
 
-def get_pcmk_log() -> str:
+def get_pcmk_log() -> Optional[str]:
     """
     Get the path of the pacemaker log file
     """
@@ -57,7 +57,7 @@ def get_pcmk_log() -> str:
             return log
 
     logger.warning("No valid pacemaker log file found")
-    return ""
+    return None
 
 
 def collect_ha_logs(context: core.Context) -> None:
@@ -65,9 +65,11 @@ def collect_ha_logs(context: core.Context) -> None:
     Collect pacemaker, corosync and extra logs
     """
     log_list = [get_pcmk_log(), get_corosync_log()] + context.extra_log_list
-    for log in log_list:
-        if log and os.path.isfile(log):
-            utils.dump_logset(context, log)
+    log_list = [os.path.expanduser(log) for log in log_list if log is not None]
+    log_list_marked_same_basename = utils.mark_duplicate_basenames(log_list)
+    for log, same_basename in log_list_marked_same_basename:
+        if os.path.isfile(log):
+            utils.dump_logset(context, log, create_dir=same_basename)
 
 
 def collect_journal_logs(context: core.Context) -> None:
@@ -116,15 +118,15 @@ def dump_D_process() -> str:
     return out_string
 
 
-def lsof_ocfs2_device() -> str:
+def lsof_cluster_fs_device(fs_type: str) -> str:
     """
-    List open files for OCFS2 device
+    List open files for OCFS2/GFS2 device
     """
     out_string = ""
 
     sh_utils_inst = ShellUtils()
     _, out, _ = sh_utils_inst.get_stdout_stderr("mount")
-    dev_list = re.findall("^(.*) on .* type ocfs2 ", out, re.MULTILINE)
+    dev_list = re.findall(f"^(.*) on .* type {fs_type.lower()} ", out, re.M)
     for dev in dev_list:
         cmd = f"lsof {dev}"
         out_string += "\n\n#=====[ Command ] ==========================#\n"
@@ -136,9 +138,9 @@ def lsof_ocfs2_device() -> str:
     return out_string
 
 
-def ocfs2_commands_output() -> str:
+def cluster_fs_commands_output(fs_type: str) -> str:
     """
-    Run ocfs2 related commands, return outputs
+    Run OCFS2/GFS2 related commands, return outputs
     """
     out_string = ""
 
@@ -146,11 +148,16 @@ def ocfs2_commands_output() -> str:
         "dmesg",
         "ps -efL",
         "lsblk -o 'NAME,KNAME,MAJ:MIN,FSTYPE,LABEL,RO,RM,MODEL,SIZE,OWNER,GROUP,MODE,ALIGNMENT,MIN-IO,OPT-IO,PHY-SEC,LOG-SEC,ROTA,SCHED,MOUNTPOINT'",
-        "mounted.ocfs2 -f",
         "findmnt",
-        "mount",
-        "cat /sys/fs/ocfs2/cluster_stack"
+        "mount"
     ]
+
+    if fs_type.lower() == "ocfs2":
+        cmds.extend([
+            "mounted.ocfs2 -f",
+            "cat /sys/fs/ocfs2/cluster_stack"
+        ])
+
     for cmd in cmds:
         cmd_name = cmd.split()[0]
         if not shutil.which(cmd_name):
@@ -164,25 +171,39 @@ def ocfs2_commands_output() -> str:
     return out_string
 
 
-def collect_ocfs2_info(context: core.Context) -> None:
+def collect_cluster_fs_info(context: core.Context) -> None:
     """
-    Collects OCFS2 information
+    Collects OCFS2 and GFS2 information
     """
-    out_string = ""
-    rc, out, err = ShellUtils().get_stdout_stderr("mounted.ocfs2 -d")
-    if rc != 0:
-        out_string += f"Failed to run \"mounted.ocfs2 -d\": {err}"
-    # No ocfs2 device, just header line printed
-    elif len(out.split('\n')) == 1:
-        out_string += "No ocfs2 partitions found"
-    else:
-        out_string += dump_D_process()
-        out_string += lsof_ocfs2_device()
-        out_string += ocfs2_commands_output()
+    def collect_info(cmd: str, fs_type: str, output_file: str) -> None:
+        out_string = ""
+        no_partition_msg = f"No {fs_type} partitions found"
 
-    ocfs2_f = os.path.join(context.work_dir, constants.OCFS2_F)
-    logger.debug(f"Dump OCFS2 information into {utils.real_path(ocfs2_f)}")
-    crmutils.str2file(out_string, ocfs2_f)
+        rc, out, err = ShellUtils().get_stdout_stderr(cmd)
+        if rc != 0:
+            if fs_type == "OCFS2":
+                error_msg = f"Failed to run \"{cmd}\": {err}"
+                out_string += error_msg
+                logger.error(error_msg)
+            elif fs_type == "GFS2":
+                out_string += no_partition_msg
+        elif fs_type == "OCFS2" and len(out.split('\n')) == 1:
+            out_string += no_partition_msg
+        else:
+            out_string += dump_D_process()
+            out_string += lsof_cluster_fs_device(fs_type)
+            out_string += cluster_fs_commands_output(fs_type)
+
+        target_f = os.path.join(context.work_dir, output_file)
+        logger.debug("Dump %s information into %s", fs_type, utils.real_path(target_f))
+        crmutils.str2file(out_string, target_f)
+
+    # Collect OCFS2 information
+    if shutil.which("mounted.ocfs2"):
+        collect_info("mounted.ocfs2 -d", "OCFS2", constants.OCFS2_F)
+
+    # Collect GFS2 information
+    collect_info('mount|grep "type gfs2"', "GFS2", constants.GFS2_F)
 
 
 def collect_ratraces(context: core.Context) -> None:
@@ -221,7 +242,7 @@ def collect_dlm_info(context: core.Context) -> None:
         name_list = []
         out_string = "##### NOTICE - Lockspace overview:\n"
         out_string += utils.get_cmd_output("dlm_tool ls")
-        name_list = re.findall("^name\s*(.*)$", out_string, re.MULTILINE)
+        name_list = re.findall(r"^name\s*(.*)$", out_string, re.MULTILINE)
 
         for name in name_list:
             out_string += f"\n\n## NOTICE - Lockspace {name}\n"
@@ -349,7 +370,7 @@ def dump_runtime_state(workdir: str) -> None:
 
     # Dump other runtime state files
     for cmd, f, desc in [
-        ("cibadmin -Ql", constants.CIB_F, "CIB contents"),
+        ("cibadmin -Q", constants.CIB_F, "CIB contents"),
         ("crm_node -p", constants.MEMBERSHIP_F, "members of this partition")
     ]:
         out = cluster_shell_inst.get_stdout_or_raise_error(cmd)
@@ -447,11 +468,16 @@ def collect_sbd_info(context: core.Context) -> None:
         return
 
     sbd_f = os.path.join(context.work_dir, constants.SBD_F)
-    cmd = ". {};export SBD_DEVICE;{};{}".format(constants.SBDCONF, "sbd dump", "sbd list")
+    cmd_list = [
+        f". {constants.SBDCONF};export SBD_DEVICE;sbd dump;sbd list",
+        "crm sbd configure show",
+        "crm sbd status"
+    ]
     with open(sbd_f, "w") as f:
-        f.write("\n\n#=====[ Command ] ==========================#\n")
-        f.write(f"# {cmd}\n")
-        f.write(utils.get_cmd_output(cmd))
+        for cmd in cmd_list:
+            f.write("\n\n#=====[ Command ] ==========================#\n")
+            f.write(f"# {cmd}\n")
+            f.write(utils.get_cmd_output(cmd))
 
     logger.debug(f"Dump SBD config file into {utils.real_path(sbd_f)}")
 
@@ -511,13 +537,16 @@ def collect_qdevice_info(context: core.Context) -> None:
     """
     Collect quorum/qdevice/qnetd information
     """
-    out_string = f"##### Quorum status #####\n"
+    service_manager = ServiceManager()
+    if not service_manager.service_is_active("corosync.service"):
+        return
+    out_string = "##### Quorum status #####\n"
     out_string += corosync.query_quorum_status() + "\n"
 
-    if ServiceManager().service_is_active("corosync-qdevice.service"):
-        out_string += f"\n##### Qdevice status #####\n"
+    if service_manager.service_is_active("corosync-qdevice.service"):
+        out_string += "\n##### Qdevice status #####\n"
         out_string += corosync.query_qdevice_status() + "\n"
-        out_string += f"\n##### Qnetd status #####\n"
+        out_string += "\n##### Qnetd status #####\n"
         out_string += corosync.query_qnetd_status() + "\n"
 
     _file = os.path.join(context.work_dir, constants.QDEVICE_F)
