@@ -40,7 +40,7 @@ class Context:
         self.to_time: float = utils.now()
         self.no_compress: bool = not config.report.compress
         self.speed_up: bool = config.report.speed_up
-        self.extra_log_list: List[str] = config.report.collect_extra_logs.split()
+        self.extra_log_list: List[str] = config.report.collect_extra_logs
         self.rm_exist_dest: bool = config.report.remove_exist_dest
         self.single: bool= config.report.single_node
         self.sensitive_regex_list: List[str] = []
@@ -173,6 +173,10 @@ def pick_first_compress():
         "bzip2": ".bz2",
         "xz": ".xz"
     }
+    default_prog = config.report.compress_prog
+    default_suffix = compress_prog_suffix_dict.get(default_prog)
+    if shutil.which(default_prog) and default_suffix:
+        return default_prog, default_suffix
     for cmd, suffix in compress_prog_suffix_dict.items():
         if shutil.which(cmd):
             return cmd, suffix
@@ -348,14 +352,7 @@ def process_node_list(context: Context) -> None:
         if not context.node_list:
             raise utils.ReportGenericError("Could not figure out a list of nodes; is this a cluster node?")
 
-    for node in context.node_list[:]:
-        if node == context.me:
-            continue
-        try:
-            crmutils.node_reachable_check(node)
-        except Exception as err:
-            logger.error(str(err))
-            context.node_list.remove(node)
+    context.node_list = crmutils.get_reachable_node_list(context.node_list)
 
 
 def process_arguments(context: Context) -> None:
@@ -402,12 +399,6 @@ def find_ssh_user(context: Context) -> None:
         if ret is not None:
             logger.debug("passwordless ssh to %s is OK", n)
             context.passwordless_shell_for_nodes[n] = ret
-        elif user_of_host.use_ssh_agent() and 'SSH_AUTH_SOCK' not in os.environ:
-            with StringIO() as buf:
-                buf.write('Environment variable SSH_AUTH_SOCK does not exist.')
-                if 'SUDO_USER' in os.environ:
-                    buf.write(' Please check whether ssh-agent is available and consider using "sudo --preserve-env=SSH_AUTH_SOCK".')
-                logger.warning('%s', buf.getvalue())
         else:
             logger.warning("passwordless ssh to node %s does not work", n)
             if not crmutils.can_ask():
