@@ -13,12 +13,14 @@ from copy import deepcopy
 from glob import glob
 from lxml import etree
 
+from .prun.runner import Task
+
 try:
     import json
 except ImportError:
     import simplejson as json
 
-from . import config
+from . import config, constants, user_of_host
 from . import handles
 from . import options
 from . import userdir
@@ -1068,15 +1070,37 @@ def _check_control_persist():
         print((".EXT", cmd))
     cmd = subprocess.Popen(cmd,
                            stdout=subprocess.PIPE,
-                           stderr=subprocess.PIPE)
+                           stderr=subprocess.PIPE,
+                           env=os.environ,  # bsc#1205925
+                           )
     (out, err) = cmd.communicate()
     return "Bad configuration option" not in err
 
 
+class _PRunAsCurrentUserInterceptor(prun.PRunInterceptor):
+    _user = userdir.getuser()
+
+    def task(self, task: Task) -> Task:
+        if self._user != 'hacluster':
+            return task
+        elif any('ssh' in arg for arg in task.args):
+            task.args = ['ssh']
+            task.args .extend(constants.SSH_OPTION_ARGS)
+            task.args .extend([task.context['host'], '/bin/bash'])
+        else:
+            task.args = ['/bin/bash']
+        task.context['ssh_user'] = self._user
+        return task
+
+
 def _parallax_call(printer, hosts, cmd, timeout_seconds):
     "parallax.call with debug logging"
     printer.debug("parallax.call(%s, %s)" % (repr(hosts), cmd))
-    return prun.prun({host: cmd for host, _, _ in hosts}, timeout_seconds=timeout_seconds)
+    return prun.prun(
+        {host: cmd for host, _, _ in hosts},
+        interceptor=_PRunAsCurrentUserInterceptor(),
+        timeout_seconds=timeout_seconds,
+    )
 
 
 def _resolve_script(name):
@@ -1090,10 +1114,29 @@ def _resolve_script(name):
     return None
 
 
+class _PRunCopyAsCurrentUserInterceptor(prun.PRunInterceptor):
+    _user = userdir.getuser()
+
+    def task(self, task: Task) -> Task:
+        if self._user != 'hacluster':
+            return task
+        task.args = ['sftp']
+        task.args.extend(constants.SSH_OPTION_ARGS)
+        task.args.extend(['-o', 'BatchMode=yes', '-b', '-', task.context['host']])
+        task.context['remote_user'] = userdir.getuser()
+        return task
+
+
 def _parallax_copy(printer, hosts, src, dst, timeout_seconds):
     "parallax.copy with debug logging"
     printer.debug("parallax.copy(%s, %s, %s)" % (repr(hosts), src, dst))
-    return prun.pcopy_to_remote(src, [x[0] for x in hosts], dst, recursive=True, timeout_seconds=timeout_seconds)
+    return prun.pcopy_to_remote(
+        src,
+        [x[0] for x in hosts],
+        dst,
+        recursive=True, timeout_seconds=timeout_seconds,
+        interceptor=_PRunCopyAsCurrentUserInterceptor(),
+    )
 
 
 def _tempname(prefix):
@@ -1138,7 +1181,11 @@ def _cleanup_local(workdir):
     if workdir and os.path.isdir(workdir):
         cleanscript = os.path.join(workdir, 'crm_clean.py')
         if os.path.isfile(cleanscript):
-            if subprocess.call([cleanscript, workdir], shell=False) != 0:
+            if subprocess.call(
+                    [cleanscript, workdir],
+                    shell=False,
+                    env=os.environ,  # bsc#1205925
+            ) != 0:
                 shutil.rmtree(workdir)
         else:
             shutil.rmtree(workdir)
@@ -1592,6 +1639,16 @@ def _copy_utils(dst):
         raise ValueError(e)
 
 
+def _check_parallax_remote_available(printer, hosts):
+    try:
+        _parallax_call(printer, hosts, 'true', timeout_seconds=15)
+    except user_of_host.UserNotFoundError:
+        if userdir.getuser() == 'hacluster':
+            raise ValueError('Passwordless ssh does not work. Run "crm cluster health hawk2 --fix" to set it up.') from None
+        else:
+            raise ValueError('Passwordless ssh does not work.') from None
+
+
 def _create_remote_workdirs(printer, hosts, path, timeout_seconds):
     "Create workdirs on remote hosts"
     ok = True
@@ -1732,6 +1789,7 @@ class RunActions(object):
             json.dump(self.data, open(self.statefile, 'w'))
             _copy_utils(self.workdir)
             if has_remote_actions:
+                _check_parallax_remote_available(self.printer, self.hosts)
                 _create_remote_workdirs(self.printer, self.hosts, self.workdir, self.timeout_seconds)
                 _copy_to_remote_dirs(self.printer, self.hosts, self.workdir, self.timeout_seconds)
             # make sure all path references are relative to the script directory
@@ -1986,7 +2044,7 @@ class RunActions(object):
         Handle an action that executes locally
         """
         if self.sudo_pass:
-            input_s = u'sudo: %s\n' % (self.sudo_pass)
+            input_s = 'sudo: %s\n' % (self.sudo_pass)
         else:
             input_s = None
         if self.dry_run:
@@ -2059,7 +2117,10 @@ def run(script, params, printer):
     finally:
         if not dry_run:
             if not config.core.debug:
-                _run_cleanup(printer, has_remote_actions, local_node, hosts, workdir, int(params['timeout']))
+                try:
+                    _run_cleanup(printer, has_remote_actions, local_node, hosts, workdir, int(params['timeout']))
+                except user_of_host.UserNotFoundError:
+                    pass
             elif has_remote_actions:
                 _print_debug(printer, local_node, hosts, workdir, int(params['timeout']))
             else:
