@@ -57,13 +57,7 @@ class AuthorizationError(Error):
         self.user = user
 
     def diagnose(self) -> str:
-        with StringIO() as buf:
-            if user_of_host.instance().use_ssh_agent():
-                if 'SSH_AUTH_SOCK' not in os.environ:
-                    buf.write('Environment variable SSH_AUTH_SOCK does not exist.')
-                    if 'SUDO_USER' in os.environ:
-                        buf.write(' Please check whether ssh-agent is available and consider using "sudo --preserve-env=SSH_AUTH_SOCK".')
-            return buf.getvalue()
+        return ''
 
 
 class NonInteractiveSSHAuthorizationError(AuthorizationError):
@@ -206,7 +200,7 @@ class LocalShell:
             user: typing.Optional[str],
             cmd: str,
             success_exit_status: typing.Optional[typing.Set[int]] = None,
-    ):
+    ) -> str:
         result = self.su_subprocess_run(
             user, cmd,
             stdout=subprocess.PIPE,
@@ -348,7 +342,6 @@ class ClusterShell:
                     host,
                     user,
                     '--preserve-env=SSH_AUTH_SOCK' if self.forward_ssh_agent else '',
-                    constants.SSH_OPTION,
                 ),
                 input=cmd.encode('utf-8'),
                 start_new_session=True,
@@ -385,6 +378,14 @@ class ClusterShell:
         rc, stdout, stderr = self.get_rc_stdout_stderr_raw_without_input(host, cmd)
         return rc, Utils.decode_str(stdout).strip(), Utils.decode_str(stderr).strip()
 
+    def get_rc_output_without_input(self, host, cmd) -> typing.Tuple[int, str]:
+        result = self.subprocess_run_without_input(
+            host, None, cmd,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT
+        )
+        return result.returncode, Utils.decode_str(result.stdout).strip()
+
     def get_stdout_or_raise_error(
             self,
             cmd: str,
@@ -418,7 +419,6 @@ class ClusterShell:
                 host,
                 user,
                 '--preserve-env=SSH_AUTH_SOCK' if self.forward_ssh_agent else '',
-                constants.SSH_OPTION,
             ),
             input=cmd.encode('utf-8'),
             **kwargs,
@@ -458,9 +458,11 @@ class ShellUtils:
         return proc.returncode, stdout_data.strip()
 
     @classmethod
-    def get_stdout_stderr(cls, cmd, input_s=None, shell=True, raw=False, no_reg=False, timeout=None):
+    def get_stdout_stderr(cls, cmd, input_s=None, shell=True, raw=False, no_reg=False, timeout=None, mix_stderr=False):
         '''
         Run a cmd, return (rc, stdout, stderr)
+        Parameters:
+            mix_stderr: whether to mix stderr data into stdout
         '''
         if crmsh.options.regression_tests and not no_reg:
             print(".EXT", cmd)
@@ -469,7 +471,7 @@ class ShellUtils:
             shell=shell,
             stdin=input_s and subprocess.PIPE or None,
             stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            stderr=subprocess.STDOUT if mix_stderr else subprocess.PIPE,
             env=os.environ,  # bsc#1205925
         )
         # will raise subprocess.TimeoutExpired if set timeout
@@ -478,9 +480,10 @@ class ShellUtils:
             return proc.returncode, stdout_data, stderr_data
         else:
             if isinstance(stdout_data, bytes):
-                stdout_data = Utils.decode_str(stdout_data)
-                stderr_data = Utils.decode_str(stderr_data)
-        return proc.returncode, stdout_data.strip(), stderr_data.strip()
+                stdout_data = Utils.decode_str(stdout_data).strip()
+            if isinstance(stderr_data, bytes):
+                stderr_data = Utils.decode_str(stderr_data).strip()
+        return proc.returncode, stdout_data, stderr_data
 
 
 class ClusterShellAdaptorForLocalShell(ClusterShell):
@@ -502,5 +505,10 @@ class ClusterShellAdaptorForLocalShell(ClusterShell):
 
 
 def cluster_shell():
-    return ClusterShell(LocalShell(), user_of_host.instance(), raise_ssh_error=True)
+    return ClusterShell(
+        LocalShell(additional_environ={'SSH_AUTH_SOCK': os.environ.get('SSH_AUTH_SOCK', '')}),
+        user_of_host.instance(),
+        forward_ssh_agent=True,
+        raise_ssh_error=True,
+    )
 
