@@ -48,11 +48,15 @@ class Key:
     def public_key(self) -> str:
         raise NotImplementedError
 
+    def fingerprint(self) -> str:
+        raise NotImplementedError
+
 
 class KeyFile(Key):
     def __init__(self, path: str):
         self._path = os.path.realpath(path)
         self._public_key = None
+        self._fingerprint = None
 
     def public_key_file(self) -> typing.Optional[str]:
         return self._path
@@ -65,6 +69,21 @@ class KeyFile(Key):
                 self._public_key = f.read().strip()
             return self._public_key
 
+    def fingerprint(self) -> str:
+        if self._fingerprint:
+            return self._fingerprint
+        else:
+            result = subprocess.run(
+                ['ssh-keygen', '-l', '-f', self.public_key_file()],
+                stdin=subprocess.DEVNULL,
+                stdout=subprocess.PIPE,
+            )
+            if result.returncode == 0:
+                self._fingerprint = result.stdout.decode('utf-8', 'backslashreplace').strip()
+                return self._fingerprint
+            else:
+                raise ValueError(f'Failed to generate fingerprint: {result.returncode}.')
+
     def __eq__(self, other):
         return isinstance(other, KeyFile) and self._path == other._path and self.public_key() == other.public_key()
 
@@ -75,10 +94,27 @@ class KeyFile(Key):
 class InMemoryPublicKey(Key):
     def __init__(self, content: str):
         self.content = content.strip()
+        self._fingerprint = None
 
     def public_key(self) -> str:
         return self.content
 
+    def fingerprint(self) -> str:
+        if self._fingerprint:
+            return self._fingerprint
+        else:
+            child = subprocess.Popen(
+                ['ssh-keygen', '-l', '-f', '/dev/stdin'],
+                stdin=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+            )
+            stdout, _ = child.communicate(self.public_key().encode('utf-8'))
+            if child.returncode == 0:
+                self._fingerprint = stdout.decode('utf-8', 'backslashreplace').strip()
+                return self._fingerprint
+            else:
+                raise ValueError(f'Failed to generate fingerprint: {child.returncode}.')
+
     def __eq__(self, other):
         return isinstance(other, InMemoryPublicKey) and self.content == other.content
 
@@ -130,8 +166,8 @@ class AuthorizedKeyManager:
                     tmp.flush()
                     self._add_by_ssh_copy_id(user, host, tmp.name)
 
-    @classmethod
-    def _add_by_editing_file(cls, user: str, key: Key):
+    @staticmethod
+    def _add_by_editing_file(user: str, key: Key):
         public_key = key.public_key()
         dir = f'~{user}/.ssh'
         file = f'{dir}/authorized_keys'
@@ -168,7 +204,9 @@ class AgentClient:
             self.socket_path = None
         else:
             self.socket_path = socket_path
-        self.shell = sh.LocalShell(additional_environ={'SSH_AUTH_SOCK': self.socket_path} if self.socket_path else None)
+        self.shell = sh.LocalShell(
+            additional_environ={'SSH_AUTH_SOCK': self.socket_path} if self.socket_path is not None else None,
+        )
 
     def list(self) -> typing.List[Key]:
         cmd = 'ssh-add -L'
@@ -183,6 +221,7 @@ class AgentClient:
 
 
 class KeyFileManager:
+    DEFAULT_KEY_TYPE = 'rsa'
     KNOWN_KEY_TYPES = ['rsa', 'ed25519', 'ecdsa']   # dsa is not listed here as it is not so secure
     KNOWN_PUBLIC_KEY_FILENAME_PATTERN = re.compile('/id_(?:{})\\.pub$'.format('|'.join(KNOWN_KEY_TYPES)))
 
@@ -208,7 +247,7 @@ class KeyFileManager:
         filenames = self.list_public_key_for_user(host, user)
         if not filenames:
             return list()
-        cmd = f'cat ~{user}/.ssh/{{{",".join(filenames)}}}'
+        cmd = f'cat {",".join(filenames)}'
         result = self.cluster_shell.subprocess_run_without_input(
             host, user,
             cmd,
@@ -231,8 +270,9 @@ class KeyFileManager:
         * is_generated: whether a new keypair is generated
         * list_of_public_keys: all public keys of known types, including the newly generated one
         """
-        script = '''if [ ! \\( {condition} \\) ]; then
-    ssh-keygen -t rsa -f ~/.ssh/id_rsa -q -C "Cluster internal on $(hostname)" -N '' <> /dev/null
+        script = '''set -e
+if [ ! \\( {condition} \\) ]; then
+    ssh-keygen -t {key_type} -f ~/.ssh/id_{key_type} -q -C "Cluster internal on $(hostname)" -N '' <> /dev/null
     echo 'GENERATED=1'
 fi
 for file in ~/.ssh/id_{{{pattern}}}; do
@@ -245,6 +285,7 @@ for file in ~/.ssh/id_{{{pattern}}}; do
 done
 '''.format(
             condition=' -o '.join([f'-f ~/.ssh/id_{t}' for t in self.KNOWN_KEY_TYPES]),
+            key_type=self.DEFAULT_KEY_TYPE,
             pattern=','.join(self.KNOWN_KEY_TYPES),
         )
         result = self.cluster_shell.subprocess_run_without_input(
@@ -265,3 +306,58 @@ done
             else:
                 keys.append(InMemoryPublicKey(line))
         return generated, keys
+
+
+def fetch_public_key_file_list(
+        host: typing.Optional[str],
+        user: str,
+        generate_key_pair: bool = False
+) -> typing.List[KeyFile]:
+    """
+    Fetch the public key file list for the specified user on the specified host.
+
+    :param host: the host where the user is located. If None, the local host is assumed.
+    :param user: the user name
+    :param generate_key_pair: whether to generate a new key pair if no key pair is found,
+     default is False
+
+    :return: a list of public key file paths
+
+    :raise Error: if no public key file is found for the user
+    """
+    key_file_manager = KeyFileManager(sh.cluster_shell())
+    if generate_key_pair:
+        key_file_manager.ensure_key_pair_exists_for_user(host, user)
+    public_keys = key_file_manager.list_public_key_for_user(host, user)
+    if not public_keys:
+        host_str = f'@{host}' if host else ' locally'
+        raise Error(f'No public key file found for {user}{host_str}')
+    return [KeyFile(p) for p in public_keys]
+
+
+def fetch_public_key_content_list(
+        host: typing.Optional[str],
+        user: str,
+        generate_key_pair: bool = False
+) -> typing.List[str]:
+    """
+    Fetch the public key content list for the specified user on the specified host.
+
+    :param host: the host where the user is located. If None, the local host is assumed.
+    :param user: the user name
+    :param generate_key_pair: whether to generate a new key pair if no key pair is found,
+     default is False
+
+    :return: a list of public key strings
+
+    :raise Error: if no public key file is found for the user
+    """
+    key_file_manager = KeyFileManager(sh.cluster_shell())
+    if generate_key_pair:
+        key_file_manager.ensure_key_pair_exists_for_user(host, user)
+    keys_in_memory = key_file_manager.load_public_keys_for_user(host, user)
+    public_keys = [key.public_key() for key in keys_in_memory]
+    if not public_keys:
+        host_str = f'@{host}' if host else ' locally'
+        raise Error(f'No public key file found for {user}{host_str}')
+    return public_keys
