@@ -1,9 +1,8 @@
 # Copyright (C) 2008-2011 Dejan Muhamedagic <dmuhamedagic@suse.de>
 # See COPYING for license information.
-
+import os
 import sys
 import re
-
 # from: http://code.activestate.com/recipes/475116/
 
 # A module that can be used to portably generate formatted output to
@@ -62,6 +61,7 @@ class colors(object):
     RLIGNOREBEGIN = '\001'
     RLIGNOREEND = '\002'
 
+
 _STRING_CAPABILITIES = """
 BOL=cr UP=cuu1 DOWN=cud1 LEFT=cub1 RIGHT=cuf1
 CLEAR_SCREEN=clear CLEAR_EOL=el CLEAR_BOL=el1 CLEAR_EOS=ed BOLD=bold
@@ -74,7 +74,10 @@ _ANSICOLORS = "BLACK RED GREEN YELLOW BLUE MAGENTA CYAN WHITE".split()
 
 def _tigetstr(cap_name):
     import curses
-    cap = curses.tigetstr(cap_name) or ''
+    if not curses.tigetstr(cap_name):
+        return None
+    from .utils import to_ascii
+    cap = to_ascii(curses.tigetstr(cap_name)) or ''
 
     # String capabilities can include "delays" of the form "$<2>".
     # For any modern terminal, we should be able to just ignore
@@ -101,6 +104,7 @@ def _tigetstr(cap_name):
 
 def _lookup_caps():
     import curses
+    from .utils import to_ascii
 
     # Look up numeric capabilities.
     colors.COLS = curses.tigetnum('cols')
@@ -112,20 +116,20 @@ def _lookup_caps():
     # Colors
     set_fg = _tigetstr('setf')
     if set_fg:
-        for i, color in zip(range(len(_COLORS)), _COLORS):
-            setattr(colors, color, curses.tparm(set_fg, i) or '')
+        for i, color in zip(list(range(len(_COLORS))), _COLORS):
+            setattr(colors, color, to_ascii(curses.tparm(set_fg.encode('utf-8'), i)) or '')
     set_fg_ansi = _tigetstr('setaf')
     if set_fg_ansi:
-        for i, color in zip(range(len(_ANSICOLORS)), _ANSICOLORS):
-            setattr(colors, color, curses.tparm(set_fg_ansi, i) or '')
+        for i, color in zip(list(range(len(_ANSICOLORS))), _ANSICOLORS):
+            setattr(colors, color, to_ascii(curses.tparm(set_fg_ansi.encode('utf-8'), i)) or '')
     set_bg = _tigetstr('setb')
     if set_bg:
-        for i, color in zip(range(len(_COLORS)), _COLORS):
-            setattr(colors, 'BG_'+color, curses.tparm(set_bg, i) or '')
+        for i, color in zip(list(range(len(_COLORS))), _COLORS):
+            setattr(colors, 'BG_'+color, to_ascii(curses.tparm(set_bg.encode('utf-8'), i)) or '')
     set_bg_ansi = _tigetstr('setab')
     if set_bg_ansi:
-        for i, color in zip(range(len(_ANSICOLORS)), _ANSICOLORS):
-            setattr(colors, 'BG_'+color, curses.tparm(set_bg_ansi, i) or '')
+        for i, color in zip(list(range(len(_ANSICOLORS))), _ANSICOLORS):
+            setattr(colors, 'BG_'+color, to_ascii(curses.tparm(set_bg_ansi.encode('utf-8'), i)) or '')
 
 
 def init():
@@ -147,6 +151,7 @@ def init():
     from . import config
     if not _term_stream.isatty() and 'color-always' not in config.color.style:
         return
+    _ignore_environ()
     # Check the terminal type.  If we fail, then assume that the
     # terminal has no capabilities.
     try:
@@ -173,4 +178,12 @@ def is_color(s):
     return hasattr(colors, s.upper())
 
 
+def _ignore_environ():
+    """Ignore environment variable COLUMNS and ROWS"""
+    # See https://bugzilla.suse.com/show_bug.cgi?id=1205925
+    # and https://gitlab.com/procps-ng/procps/-/blob/c415fc86452c933716053a50ab1777a343190dcc/src/ps/global.c#L279
+    for name in ["COLUMNS", "ROWS"]:
+        if name in os.environ:
+            del os.environ[name]
+
 # vim:ts=4:sw=4:et:
