@@ -9,7 +9,7 @@ import typing
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
 
 import crmsh.parallax
-from . import command, sh, healthcheck
+from . import command, sh, healthcheck, migration
 from . import utils
 from . import scripts
 from . import completers as compl
@@ -162,14 +162,14 @@ class Cluster(command.UI):
         '''
         Starts the cluster stack on all nodes or specific node(s)
         '''
+        node_list = parse_option_for_nodes(context, *args)
         service_check_list = ["pacemaker.service"]
         start_qdevice = False
-        if utils.is_qdevice_configured():
+        if corosync.is_qdevice_configured():
             start_qdevice = True
             service_check_list.append("corosync-qdevice.service")
 
         service_manager = ServiceManager()
-        node_list = parse_option_for_nodes(context, *args)
         try:
             for node in node_list[:]:
                 if all([service_manager.service_is_active(srv, remote_addr=node) for srv in service_check_list]):
@@ -178,17 +178,26 @@ class Cluster(command.UI):
         except utils.NoSSHError as msg:
             logger.error('%s', msg)
             logger.info("Please try 'crm cluster start' on each node")
-            return
+            return False
         if not node_list:
             return
 
         if start_qdevice:
             service_manager.start_service("corosync-qdevice", node_list=node_list)
-        node_list = bootstrap.start_pacemaker(node_list)
-        if start_qdevice:
-            qdevice.QDevice.check_qdevice_vote()
+
+        success_flag = True
+        success_list = bootstrap.start_pacemaker(node_list=node_list)
         for node in node_list:
-            logger.info("The cluster stack started on {}".format(node))
+            if node not in success_list:
+                success_flag = False
+                logger.error("The cluster stack failed to start on %s", node)
+                continue
+            logger.info("The cluster stack started on %s", node)
+
+        if start_qdevice and success_list:
+            qdevice.QDevice.check_qdevice_vote()
+
+        return success_flag
 
     @staticmethod
     def _node_ready_to_stop_cluster_service(node):
@@ -219,22 +228,6 @@ class Cluster(command.UI):
 
         return True
 
-    @staticmethod
-    def _wait_for_dc(node=None):
-        """
-        Wait for the cluster's DC to become available
-        """
-        if not ServiceManager().service_is_active("pacemaker.service", remote_addr=node):
-            return
-
-        dc_deadtime = utils.get_property("dc-deadtime", peer=node) or str(constants.DC_DEADTIME_DEFAULT)
-        dc_timeout = int(dc_deadtime.strip('s')) + 5
-        try:
-            utils.check_function_with_timeout(utils.get_dc, wait_timeout=dc_timeout, peer=node)
-        except TimeoutError:
-            logger.error("No DC found currently, please wait if the cluster is still starting")
-            raise utils.TerminateSubCommand
-
     @staticmethod
     def _set_dlm(node=None):
         """
@@ -256,12 +249,17 @@ class Cluster(command.UI):
         except utils.NoSSHError as msg:
             logger.error('%s', msg)
             logger.info("Please try 'crm cluster stop' on each node")
-            return
+            return False
         if not node_list:
             return
         logger.debug(f"stop node list: {node_list}")
 
-        self._wait_for_dc(node_list[0])
+        if utils.is_cluster_in_maintenance_mode() and utils.is_dlm_running():
+            logger.info("The cluster is in maintenance mode")
+            logger.error("Stopping pacemaker/corosync will trigger unexpected node fencing when 'dlm_controld' is running in maintenance mode.")
+            return False
+
+        utils.wait_for_dc(node_list[0])
 
         self._set_dlm(node_list[0])
 
@@ -277,14 +275,17 @@ class Cluster(command.UI):
         for node in node_list:
             logger.info("The cluster stack stopped on {}".format(node))
 
+        return True
+
     @command.skill_level('administrator')
     def do_restart(self, context, *args):
         '''
         Restarts the cluster stack on all nodes or specific node(s)
         '''
-        parse_option_for_nodes(context, *args)
-        self.do_stop(context, *args)
-        self.do_start(context, *args)
+        stop_rc = self.do_stop(context, *args)
+        if stop_rc is False:
+            return False
+        return self.do_start(context, *args)
 
     @command.skill_level('administrator')
     def do_enable(self, context, *args):
@@ -294,7 +295,7 @@ class Cluster(command.UI):
         node_list = parse_option_for_nodes(context, *args)
         service_manager = ServiceManager()
         node_list = service_manager.enable_service("pacemaker.service", node_list=node_list)
-        if service_manager.service_is_available("corosync-qdevice.service") and utils.is_qdevice_configured():
+        if service_manager.service_is_available("corosync-qdevice.service") and corosync.is_qdevice_configured():
             service_manager.enable_service("corosync-qdevice.service", node_list=node_list)
         for node in node_list:
             logger.info("Cluster services enabled on %s", node)
@@ -338,13 +339,13 @@ option.""", usage="init [options] [STAGE]", epilog="""
 
 Stage can be one of:
     ssh         Create SSH keys for passwordless SSH between cluster nodes
+    firewalld   Add high-availability service to firewalld
     csync2      Configure csync2
     corosync    Configure corosync
     sbd         Configure SBD (requires -s <dev>)
     cluster     Bring the cluster online
     ocfs2       Configure OCFS2 (requires -o <dev>) NOTE: this is a Technical Preview
-    vgfs        Create volume group and filesystem (ocfs2 template only,
-                    requires -o <dev>) NOTE: this stage is an alias of ocfs2 stage
+    gfs2        Configure GFS2 (requires -g <dev>) NOTE: this is a Technical Preview
     admin       Create administration virtual IP (optional)
     qdevice     Configure qdevice and qnetd
 
@@ -375,9 +376,15 @@ Examples:
   # Setup the cluster on the current node, with SBD+OCFS2
   crm cluster init -s <share disk1> -o <share disk2> -y
 
+  # Setup the cluster on the current node, with SBD+GFS2
+  crm cluster init -s <share disk1> -g <share disk2> -y
+
   # Setup the cluster on the current node, with SBD+OCFS2+Cluster LVM
   crm cluster init -s <share disk1> -o <share disk2> -o <share disk3> -C -y
 
+  # Setup the cluster on the current node, with SBD+GFS2+Cluster LVM
+  crm cluster init -s <share disk1> -g <share disk2> -g <share disk3> -C -y
+
   # Add SBD on a running cluster
   crm cluster init sbd -s <share disk> -y
 
@@ -392,6 +399,9 @@ Examples:
 
   # Add OCFS2+Cluster LVM on a running cluster
   crm cluster init ocfs2 -o <share disk1> -o <share disk2> -C -y
+
+  # Add GFS2+Cluster LVM on a running cluster
+  crm cluster init gfs2 -g <share disk1> -g <share disk2> -C -y
 """, add_help=False, formatter_class=RawDescriptionHelpFormatter)
 
         parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
@@ -409,22 +419,15 @@ Examples:
                             help="Use the given watchdog device or driver name")
         parser.add_argument("-x", "--skip-csync2-sync", dest="skip_csync2", action="store_true",
                             help="Skip csync2 initialization (an experimental option)")
-        parser.add_argument("--no-overwrite-sshkey", action="store_true", dest="no_overwrite_sshkey",
-                            help='Avoid "/root/.ssh/id_rsa" overwrite if "-y" option is used (False by default; Deprecated)')
-        parser.add_argument('--use-ssh-agent', action='store_true', dest='use_ssh_agent',
-                            help="Use an existing key from ssh-agent instead of creating new key pairs")
+        parser.add_argument('--use-ssh-agent', action=argparse.BooleanOptionalAction, dest='use_ssh_agent', default=True,
+                            help="Try to use an existing key from ssh-agent (default)")
 
         network_group = parser.add_argument_group("Network configuration", "Options for configuring the network and messaging layer.")
-        network_group.add_argument("-i", "--interface", dest="nic_list", metavar="IF", action=CustomAppendAction, choices=utils.interface_choice(), default=[],
-                                   help="Bind to IP address on interface IF. Use -i second time for second interface")
-        network_group.add_argument("-u", "--unicast", action="store_true", dest="unicast",
-                                   help="Configure corosync to communicate over unicast(udpu). This is the default transport type")
-        network_group.add_argument("-U", "--multicast", action="store_true", dest="multicast",
-                                   help="Configure corosync to communicate over multicast. Default is unicast")
+        network_group.add_argument("-i", "--interface", dest="nic_addr_list", metavar="IF", action=CustomAppendAction, default=[], help=constants.INTERFACE_HELP)
+        network_group.add_argument("-t", "--transport", dest="transport", metavar="TRANSPORT", default="knet", choices=['knet', 'udpu', 'udp'],
+                help="The transport mechanism. Allowed value is knet(kronosnet)/udpu(unicast)/udp(multicast). Default is knet")
         network_group.add_argument("-A", "--admin-ip", dest="admin_ip", metavar="IP",
                                    help="Configure IP address as an administration virtual IP")
-        network_group.add_argument("-M", "--multi-heartbeats", action="store_true", dest="second_heartbeat",
-                                   help="Configure corosync with second heartbeat line")
         network_group.add_argument("-I", "--ipv6", action="store_true", dest="ipv6",
                                    help="Configure corosync use IPv6")
 
@@ -449,10 +452,12 @@ Examples:
                                    help="Block device to use for SBD fencing, use \";\" as separator or -s multiple times for multi path (up to 3 devices)")
         storage_group.add_argument("-o", "--ocfs2-device", dest="ocfs2_devices", metavar="DEVICE", action=CustomAppendAction, default=[],
                 help="Block device to use for OCFS2; When using Cluster LVM2 to manage the shared storage, user can specify one or multiple raw disks, use \";\" as separator or -o multiple times for multi path (must specify -C option) NOTE: this is a Technical Preview")
+        storage_group.add_argument("-g", "--gfs2-device", dest="gfs2_devices", metavar="DEVICE", action=CustomAppendAction, default=[],
+                help="Block device to use for GFS2; When using Cluster LVM2 to manage the shared storage, user can specify one or multiple raw disks, use \";\" as separator or -g multiple times for multi path (must specify -C option) NOTE: this is a Technical Preview")
         storage_group.add_argument("-C", "--cluster-lvm2", action="store_true", dest="use_cluster_lvm2",
-                help="Use Cluster LVM2 (only valid together with -o option) NOTE: this is a Technical Preview")
+                help="Use Cluster LVM2 (only valid together with -o or -g option) NOTE: this is a Technical Preview")
         storage_group.add_argument("-m", "--mount-point", dest="mount_point", metavar="MOUNT", default="/srv/clusterfs",
-                help="Mount point for OCFS2 device (default is /srv/clusterfs, only valid together with -o option) NOTE: this is a Technical Preview")
+                help="Mount point for OCFS2 or GFS2 device (default is /srv/clusterfs, only valid together with -o or -g option) NOTE: this is a Technical Preview")
 
         options, args = parse_options(parser, args)
         if options is None or args is None:
@@ -461,9 +466,6 @@ Examples:
         stage = ""
         if len(args):
             stage = args[0]
-        if stage == "vgfs":
-            stage = "ocfs2"
-            logger.warning("vgfs stage was deprecated and is an alias of ocfs2 stage now")
 
         if options.qnetd_addr_input:
             if not ServiceManager().service_is_available("corosync-qdevice.service"):
@@ -483,7 +485,7 @@ Examples:
         boot_context.cluster_is_running = ServiceManager(sh.ClusterShellAdaptorForLocalShell(sh.LocalShell())).service_is_active("pacemaker.service")
         boot_context.type = "init"
         boot_context.initialize_qdevice()
-        boot_context.validate_option()
+        boot_context.validate()
 
         bootstrap.bootstrap_init(boot_context)
         bootstrap.bootstrap_add(boot_context)
@@ -503,6 +505,7 @@ to the -c option.""",usage="join [options] [STAGE]", epilog="""
 
 Stage can be one of:
     ssh         Obtain SSH keys from existing cluster node (requires -c <host>)
+    firewalld   Add high-availability service to firewalld
     csync2      Configure csync2 (requires -c <host>)
     ssh_merge   Merge root's SSH known_hosts across all nodes (csync2 must
                 already be configured).
@@ -520,17 +523,15 @@ Examples:
         parser.add_argument("-h", "--help", action="store_true", dest="help", help="Show this help message")
         parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
         parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
-        parser.add_argument("-w", "--watchdog", dest="watchdog", metavar="WATCHDOG", help="Use the given watchdog device")
-        parser.add_argument('--use-ssh-agent', action='store_true', dest='use_ssh_agent',
-                            help="Use an existing key from ssh-agent instead of creating new key pairs")
+        parser.add_argument('--use-ssh-agent', action=argparse.BooleanOptionalAction, dest='use_ssh_agent', default=True,
+                            help="Try to use an existing key from ssh-agent (default)")
 
         network_group = parser.add_argument_group("Network configuration", "Options for configuring the network and messaging layer.")
         network_group.add_argument(
             "-c", "--cluster-node", metavar="[USER@]HOST", dest="cluster_node",
             help="User and host to login to an existing cluster node. The host can be specified with either a hostname or an IP.",
         )
-        network_group.add_argument("-i", "--interface", dest="nic_list", metavar="IF", action=CustomAppendAction, choices=utils.interface_choice(), default=[],
-                help="Bind to IP address on interface IF. Use -i second time for second interface")
+        network_group.add_argument("-i", "--interface", dest="nic_addr_list", metavar="IF", action=CustomAppendAction, default=[], help=constants.INTERFACE_HELP)
         options, args = parse_options(parser, args)
         if options is None or args is None:
             return
@@ -544,7 +545,7 @@ Examples:
         join_context.stage = stage
         join_context.cluster_is_running = ServiceManager(sh.ClusterShellAdaptorForLocalShell(sh.LocalShell())).service_is_active("pacemaker.service")
         join_context.type = "join"
-        join_context.validate_option()
+        join_context.validate()
 
         bootstrap.bootstrap_join(join_context)
 
@@ -623,7 +624,7 @@ Finally run `crm cluster init qdevice` on any node in the cluster to re-deploy t
         if not cib_factory.commit():
             context.fatal_error("Change property cluster-name failed!")
 
-        if xmlutil.CrmMonXmlParser().is_any_resource_running():
+        if xmlutil.CrmMonXmlParser().is_non_stonith_resource_running():
             context.info("To apply the change, restart the cluster service at convenient time")
         else:
             bootstrap.restart_cluster()
@@ -738,8 +739,8 @@ an existing cluster.""",
         parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
         parser.add_argument("-c", "--cluster-node", metavar="[USER@]HOST", help="An already-configured geo cluster or arbitrator", dest="cluster_node")
         parser.add_argument("-s", "--clusters", help="Geo cluster description (see geo-init for details)", dest="clusters", metavar="DESC")
-        parser.add_argument('--use-ssh-agent', action='store_true', dest='use_ssh_agent',
-                            help="Use an existing key from ssh-agent instead of creating new key pairs")
+        parser.add_argument('--use-ssh-agent', action=argparse.BooleanOptionalAction, dest='use_ssh_agent', default=True,
+                            help="Try to use an existing key from ssh-agent (default)")
         options, args = parse_options(parser, args)
         if options is None or args is None:
             return
@@ -777,8 +778,8 @@ to get the geo cluster configuration.""",
         parser.add_argument("-q", "--quiet", help="Be quiet (don't describe what's happening, just do it)", action="store_true", dest="quiet")
         parser.add_argument("-y", "--yes", help='Answer "yes" to all prompts (use with caution)', action="store_true", dest="yes_to_all")
         parser.add_argument("-c", "--cluster-node", metavar="[USER@]HOST", help="An already-configured geo cluster", dest="cluster_node")
-        parser.add_argument('--use-ssh-agent', action='store_true', dest='use_ssh_agent',
-                            help="Use an existing key from ssh-agent instead of creating new key pairs")
+        parser.add_argument('--use-ssh-agent', action=argparse.BooleanOptionalAction, dest='use_ssh_agent', default=True,
+                            help="Try to use an existing key from ssh-agent (default)")
         options, args = parse_options(parser, args)
         if options is None or args is None:
             return
@@ -789,45 +790,64 @@ to get the geo cluster configuration.""",
         bootstrap.bootstrap_arbitrator(geo_context)
         return True
 
-    @command.completers(compl.choice(['hawk2']))
+    @command.completers(compl.choice([
+        'hawk2',
+        'sles16',
+    ]))
     def do_health(self, context, *args):
         '''
         Extensive health check.
         '''
         if not args:
             return self._do_health_legacy(context, *args)
-        parser = argparse.ArgumentParser()
-        parser.add_argument('component', choices=['hawk2'])
+        parser = argparse.ArgumentParser('health')
+        parser.add_argument('component', choices=['hawk2', 'sles16'])
         parser.add_argument('-f', '--fix', action='store_true')
-        parsed_args = parser.parse_args(args)
-        if parsed_args.component == 'hawk2':
-            nodes = utils.list_cluster_nodes()
-            if parsed_args.fix:
-                if not healthcheck.feature_full_check(healthcheck.PasswordlessPrimaryUserAuthenticationFeature(), nodes):
+        parsed_args, remaining_args = parser.parse_known_args(args)
+        match parsed_args.component:
+            case 'hawk2':
+                if remaining_args:
+                    logger.error('Known arguments: %s', ' '.join(remaining_args))
+                    return False
+                nodes = utils.list_cluster_nodes()
+                if parsed_args.fix:
+                    if not healthcheck.feature_full_check(healthcheck.PasswordlessPrimaryUserAuthenticationFeature(), nodes):
+                        try:
+                            healthcheck.feature_fix(
+                                healthcheck.PasswordlessPrimaryUserAuthenticationFeature(),
+                                nodes,
+                                utils.ask,
+                            )
+                        except healthcheck.FixFailure:
+                            logger.error('Cannot fix automatically.')
+                            return False
                     try:
-                        healthcheck.feature_fix(
-                            healthcheck.PasswordlessPrimaryUserAuthenticationFeature(),
-                            nodes,
-                            utils.ask,
-                        )
+                        healthcheck.feature_fix(healthcheck.PasswordlessHaclusterAuthenticationFeature(), nodes, utils.ask)
+                        logger.info("hawk2: passwordless ssh authentication: OK.")
+                        return True
                     except healthcheck.FixFailure:
-                        logger.error('Cannot fix automatically.')
+                        logger.error("hawk2: passwordless ssh authentication: FAIL.")
                         return False
-                try:
-                    healthcheck.feature_fix(healthcheck.PasswordlessHaclusterAuthenticationFeature(), nodes, utils.ask)
-                    logger.info("hawk2: passwordless ssh authentication: OK.")
-                    return True
-                except healthcheck.FixFailure:
-                    logger.error("hawk2: passwordless ssh authentication: FAIL.")
-                    return False
-            else:
-                if healthcheck.feature_full_check(healthcheck.PasswordlessHaclusterAuthenticationFeature(), nodes):
-                    logger.info("hawk2: passwordless ssh authentication: OK.")
-                    return True
                 else:
-                    logger.error("hawk2: passwordless ssh authentication: FAIL.")
-                    logger.warning('Please run "crm cluster health hawk2 --fix"')
+                    if healthcheck.feature_full_check(healthcheck.PasswordlessHaclusterAuthenticationFeature(), nodes):
+                        logger.info("hawk2: passwordless ssh authentication: OK.")
+                        return True
+                    else:
+                        logger.error("hawk2: passwordless ssh authentication: FAIL.")
+                        logger.warning('Please run "crm cluster health hawk2 --fix"')
+                        return False
+            case 'sles16':
+                try:
+                    if parsed_args.fix:
+                        migration.migrate(['sles16'] + remaining_args)
+                    else:
+                        return 0 == migration.check(['sles16'] + remaining_args)
+                except migration.MigrationFailure as e:
+                    logger.error('%s', e)
                     return False
+            case _:
+                logger.error('Unknown component: %s', parsed_args.component)
+                return False
 
     def _do_health_legacy(self, context, *args):
         params = self._args_implicit(context, args, 'nodes')
@@ -875,13 +895,7 @@ to get the geo cluster configuration.""",
         '''
         Execute the given command on all nodes/specific node(s), report outcome
         '''
-        if nodes:
-            hosts = list(nodes)
-        else:
-            hosts = utils.list_cluster_nodes()
-            if hosts is None:
-                context.fatal_error("failed to get node list from cluster")
-
+        hosts = utils.validate_and_get_reachable_nodes(nodes, all_nodes=True)
         for host, result in prun.prun({x: cmd for x in hosts}).items():
             if isinstance(result, prun.PRunError):
                 logger.error("[%s]: %s", host, result)
