@@ -2,8 +2,11 @@
 # Copyright (C) 2013 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
 
+import os
 import re
 import time
+import difflib
+from packaging import version
 from . import command
 from . import completers as compl
 from . import config
@@ -86,7 +89,7 @@ def _group_completer(args):
 
 def _advanced_completer(args):
     '''
-    meta completers for group/ms/clone resource type 
+    meta completers for group/clone resource type
     '''
     key_words = ["meta", "params"]
     completing = args[-1]
@@ -101,8 +104,6 @@ def _advanced_completer(args):
             return_list = utils.filter_keys(constants.group_meta_attributes, args)
         if resource_type == "clone":
             return_list = utils.filter_keys(constants.clone_meta_attributes, args)
-        if resource_type in ["ms", "master"]:
-            return_list = utils.filter_keys(constants.ms_meta_attributes, args)
     return return_list + key_words
 
 
@@ -191,6 +192,23 @@ def ra_agent_for_cpt(cpt):
     return agent
 
 
+def schema_completer(args):
+    complete_results = []
+
+    directory = '/usr/share/pacemaker/'
+    version_pattern = re.compile(r'^pacemaker-(\d+\.\d+)')
+    for filename in os.listdir(directory):
+        if version_pattern.match(filename):
+            complete_results.append(filename.strip('.rng'))
+
+    if complete_results:
+        # Sort files using packaging.version
+        complete_results.sort(key=lambda x: version.parse(version_pattern.match(x).group(1)))
+        command.enable_custom_sort_order()
+
+    return complete_results
+
+
 class CompletionHelp(object):
     '''
     Print some help on whatever last word in the line.
@@ -231,9 +249,7 @@ def _prim_params_completer(agent, args):
         return []
     elif '=' in completing:
         return []
-    if command.sort_completion_inst is not None:
-        # Set the value to 0 to use the costum sort order
-        command.sort_completion_inst.value = 0
+    command.enable_custom_sort_order()
     return utils.filter_keys(agent.params(), args)
 
 
@@ -241,9 +257,14 @@ def _prim_meta_completer(agent, args):
     completing = args[-1]
     if completing == 'meta':
         return ['meta']
+    if completing.endswith('='):
+        if len(completing) > 1 and options.interactive:
+            topic = completing[:-1]
+            CompletionHelp.help(topic, agent.meta_parameter(topic), args)
+        return []
     if '=' in completing:
         return []
-    return utils.filter_keys(constants.rsc_meta_attributes, args)
+    return utils.filter_keys(ra.get_resource_meta_list(), args)
 
 
 def _prim_op_completer(agent, args):
@@ -307,6 +328,11 @@ def _property_completer(args):
     return _prim_params_completer(agent, args)
 
 
+def _rsc_meta_completer(args):
+    agent = ra.get_resource_meta()
+    return _prim_meta_completer(agent, args)
+
+
 def primitive_complete_complex(args):
     '''
     This completer depends on the content of the line, i.e. on
@@ -337,6 +363,8 @@ def primitive_complete_complex(args):
     if last_keyw is None:
         return []
 
+    if last_keyw == 'meta':
+        agent = ra.get_resource_meta()
     complete_results = completers_set[last_keyw](agent, args)
     if len(args) > 4 and '=' in args[-1]:
         return complete_results + keywords
@@ -531,12 +559,6 @@ class CibConfig(command.UI):
     def do_showobjects(self, context):
         cib_factory.showobjects()
 
-    @command.name('_keywords')
-    @command.skill_level('administrator')
-    def do_keywords(self, context):
-        for k, v in sorted(iter(constants.keywords.items()), key=lambda v: v[0].lower()):
-            print("%-16s %s" % (k, v))
-
     @command.level(ui_ra.RA)
     def do_ra(self):
         pass
@@ -561,6 +583,20 @@ class CibConfig(command.UI):
     def do_assist(self):
         pass
 
+    def _show_diff(self):
+        obj_orig = mkset_obj("orig")
+        obj_orig_str = obj_orig.repr()
+        obj_changed = mkset_obj()
+        obj_changed_str = obj_changed.repr()
+
+        diff = difflib.unified_diff(obj_orig_str.splitlines(), obj_changed_str.splitlines())
+        diff = [
+                line for line in diff
+                if not line.startswith('+++ ') and not line.startswith('--- ')
+        ]
+
+        utils.page_string('\n'.join(diff))
+
     @command.skill_level('administrator')
     @command.completers_repeating(_id_show_list)
     def do_show(self, context, *args):
@@ -573,10 +609,13 @@ class CibConfig(command.UI):
             #   1. "pattern1 pattern2 pattern3"
             #   2. "pattern1|pattern2|pattern3"
             # regrex here also filter out possible spaces
-            osargs = re.split('\s*\|\s*|\s+', config.core.obscure_pattern.strip('|'))
+            osargs = re.split(r'\s*\|\s*|\s+', config.core.obscure_pattern.strip('|'))
         args = [arg for arg in args if not arg.startswith('obscure:')]
         cib_factory.ensure_cib_updated()
         with obscure(osargs):
+            if args and args[0] == "changed":
+                self._show_diff()
+                return True
             set_obj = mkset_obj(*args)
             return set_obj.show()
 
@@ -734,13 +773,12 @@ class CibConfig(command.UI):
             set_obj = mkset_obj(*args)
         return set_obj.edit()
 
-    def _verify(self, set_obj_semantic, set_obj_all):
+    def _verify(self, set_obj_semantic, set_obj_all) -> utils.VerifyResult:
         rc1 = set_obj_all.verify()
+        rc2 = utils.VerifyResult.SUCCESS
         if config.core.check_frequency != "never":
             rc2 = set_obj_semantic.semantic_check(set_obj_all)
-        else:
-            rc2 = 0
-        return rc1 and rc2 <= 1
+        return rc1 | rc2
 
     @command.skill_level('administrator')
     def do_verify(self, context):
@@ -748,7 +786,8 @@ class CibConfig(command.UI):
         utils.load_cib_file_env()
         cib_factory.ensure_cib_updated()
         set_obj_all = mkset_obj("xml")
-        return self._verify(set_obj_all, set_obj_all)
+        verify_result = self._verify(set_obj_all, set_obj_all)
+        return bool(verify_result)
 
     @command.name('validate-all')
     @command.alias('validate_all')
@@ -859,7 +898,7 @@ class CibConfig(command.UI):
         argl = [x for x in argl if x not in ('-f', '--force')]
         if arg_force or config.core.force:
             if self._stop_if_running(argl) > 0:
-                utils.wait4dc(what="Stopping %s" % (", ".join(argl)))
+                utils.wait_dc_stable(what="Stopping %s" % (", ".join(argl)))
         cib_factory.ensure_cib_updated()
         return cib_factory.delete(*argl)
 
@@ -917,8 +956,12 @@ class CibConfig(command.UI):
         rc1 = True
         if replace and not force:
             rc1 = cib_factory.is_current_cib_equal()
-        rc2 = self._verify(mkset_obj("xml", "changed"), mkset_obj("xml"))
-        if rc1 and rc2:
+
+        verify_result = self._verify(mkset_obj("xml", "changed"), mkset_obj("xml"))
+        if utils.VerifyResult.FATAL_ERROR in verify_result:
+            return False
+
+        if rc1 and bool(verify_result):
             return cib_factory.commit(replace=replace)
         if force or config.core.force:
             logger.info("commit forced")
@@ -945,16 +988,21 @@ class CibConfig(command.UI):
     @command.skill_level('administrator')
     @command.completers(compl.choice(['force']))
     def do_upgrade(self, context, force=None):
-        "usage: upgrade [force]"
-        if force and force != "force":
-            context.fatal_error("Expected 'force' or no argument")
+        "usage: upgrade <force>"
+        if (not force or force != "force") and not config.core.force:
+            context.fatal_error("'force' option is required")
         cib_factory.ensure_cib_updated()
-        return cib_factory.upgrade_validate_with(force=config.core.force or force)
+        if cib_factory.upgrade_validate_with(force or config.core.force):
+            logger.info("Current schema version is %s", cib_factory.get_schema(refresh=True))
+            return True
+        return False
 
     @command.skill_level('administrator')
+    @command.completers(schema_completer)
     def do_schema(self, context, schema_st=None):
         "usage: schema [<schema>]"
-        if not schema_st:
+        utils.load_cib_file_env()
+        if not schema_st and cib_factory.is_cib_sane():
             print(cib_factory.get_schema())
             return True
         return cib_factory.change_schema(schema_st)
@@ -1049,18 +1097,6 @@ class CibConfig(command.UI):
         [meta <attribute>=<value> [<attribute>=<value>...]]"""
         return self.__conf_object(context.get_command_name(), *args)
 
-    @command.alias('master')
-    @command.skill_level('administrator')
-    @command.completers_repeating(compl.attr_id, _f_children_id_list, _advanced_completer)
-    def do_ms(self, context, *args):
-        """usage: ms <name> <rsc>
-        [params <param>=<value> [<param>=<value>...]]
-        [meta <attribute>=<value> [<attribute>=<value>...]]"""
-        format_str = " " if "meta" in args else " meta "
-        new_cmd_str = ' '.join(args) + "{}promotable=true".format(format_str)
-        logger.warning('"ms" is deprecated. Please use "clone {}"'.format(new_cmd_str))
-        return self.__conf_object(context.get_command_name(), *args)
-
     @command.skill_level('administrator')
     @command.completers_repeating(compl.attr_id, ui_ra.complete_class_provider_type,
                                   primitive_complete_complex)
@@ -1106,7 +1142,11 @@ class CibConfig(command.UI):
 
     @command.skill_level('administrator')
     @command.completers_repeating(compl.attr_id,
-                                  compl.call(schema.rng_attr_values, 'rsc_order', 'kind'),
+                                  compl.call(
+                                      lambda *args: [v + ":" for v in schema.rng_attr_values(*args)],
+                                      'rsc_order',
+                                      'kind'
+                                  ),
                                   top_rsc_tmpl_id_list)
     def do_order(self, context, *args):
         """usage: order <id> [kind]: <rsc>[:<action>] <rsc>[:<action>] ...
@@ -1125,10 +1165,13 @@ class CibConfig(command.UI):
     def do_property(self, context, *args):
         "usage: property [$id=<set_id>] <option>=<value>"
         self.__override_lower_level_attrs(*args)
+        if not args:
+            utils.multicolumn(ra.get_properties_list())
+            return
         return self.__conf_object(context.get_command_name(), *args)
 
     @command.skill_level('administrator')
-    @command.completers_repeating(_prim_meta_completer)
+    @command.completers_repeating(_rsc_meta_completer)
     def do_rsc_defaults(self, context, *args):
         "usage: rsc_defaults [$id=<set_id>] <option>=<value>"
         return self.__conf_object(context.get_command_name(), *args)
@@ -1223,7 +1266,7 @@ class CibConfig(command.UI):
                     rsc_l += el.node.findall("primitive")
                 else:
                     rsc_l.append(el.node)
-            elif xmlutil.is_normal_node(el.node):
+            elif xmlutil.is_member_node(el.node):
                 current = "n"
                 node_l.append(el.node.get("uname"))
             else:
@@ -1247,7 +1290,12 @@ class CibConfig(command.UI):
         if cib_factory.has_cib_changed():
             if no_questions_asked or not options.interactive:
                 ok = self._commit()
-            elif utils.ask("There are changes pending. Do you want to commit them?"):
-                ok = self._commit()
+            else:
+                print("There are changes pending:")
+                self._show_diff()
+                confirm_msg = "Do you want to commit them, or cancel the operation?"
+                rc = utils.ask(confirm_msg, cancel_option=True)
+                if rc:
+                    ok = self._commit()
         cib_factory.reset()
         return ok
