@@ -16,7 +16,7 @@ from . import main
 logger = log.setup_logger(__name__)
 logger_utils = log.LoggerUtils(logger)
 
-_NON_FUNCTIONAL_COMMANDS = {'help', 'cd', 'ls', 'quit', 'up'}
+_NON_FUNCTIONAL_COMMANDS = {'help', 'ls'}
 _NON_FUNCTIONAL_OPTIONS = {'--help', '--help-without-redirect'}
 
 class Context(object):
@@ -89,7 +89,7 @@ class Context(object):
                     entry = self.current_level()
                     if 'requires' in dir(entry) and not entry.requires():
                         self.fatal_error("Missing requirements")
-                utils.check_user_access(self.current_level().name)
+                    utils.check_user_access(self.current_level().name)
                 rv = self.execute_command() is not False
         except (ValueError, IOError) as e:
             logger.error("%s: %s", self.get_qualified_name(), e, exc_info=e)
@@ -104,7 +104,7 @@ class Context(object):
 
         # wait for dc if wait flag set
         if self._wait_for_dc:
-            return utils.wait4dc(self.command_name, not options.batch)
+            return utils.wait_dc_stable(self.command_name, not options.batch)
         return rv
 
     def complete(self, line):
@@ -114,8 +114,6 @@ class Context(object):
         A space at the end of the line is significant.
         '''
         complete_next = line.endswith(' ')
-        # if complete_next:
-        #    print >>sys.stderr, "complete_next is on"
 
         # copy current state
         prev_stack = list(self.stack)
@@ -137,7 +135,7 @@ class Context(object):
                     self.command_args = tokens
                     self.command_info = self.current_level().get_child(token)
 
-                    if not self.command_info:
+                    if not self.command_args and not self.command_info:
                         return self.current_level().get_completions()
                     if self.command_info.type == 'level':
                         self.enter_level(self.command_info.level)
@@ -257,8 +255,10 @@ class Context(object):
         self._in_transit = True
 
         entry = level()
-        if 'requires' in dir(entry) and not entry.requires():
-            self.fatal_error("Missing requirements")
+        if self.command_name not in _NON_FUNCTIONAL_COMMANDS \
+                and all(arg not in _NON_FUNCTIONAL_OPTIONS for arg in self.command_args):
+            if 'requires' in dir(entry) and not entry.requires():
+                self.fatal_error("Missing requirements")
         self.stack.append(entry)
         self.clear_readline_cache()
 
@@ -347,7 +347,10 @@ class Context(object):
         '''
         Exit from the top level
         '''
-        ok = self.current_level().end_game()
+        try:
+            ok = self.current_level().end_game()
+        except utils.TerminateSubCommand:
+            return
         if options.interactive and not options.batch:
             if constants.need_reset:
                 utils.ext_cmd("reset")
