@@ -2,6 +2,7 @@
 # Copyright (C) 2013-2018 Kristoffer Gronlund <kgronlund@suse.com>
 # See COPYING for license information.
 
+import re
 from . import command
 from . import completers as compl
 from . import constants
@@ -173,7 +174,8 @@ def set_deep_meta_attr(rsc, attr, value, commit=True):
     return True
 
 
-_attrcmds = compl.choice(['delete', 'set', 'show'])
+ATTR_OPERATION_CMDS = ('delete', 'set', 'show')
+_attrcmds = compl.choice(ATTR_OPERATION_CMDS)
 _raoperations = compl.choice(constants.ra_operations)
 
 
@@ -325,7 +327,7 @@ class RscMgmt(command.UI):
         logger.info("ordering %s to stop", ", ".join(resources))
         if not self._commit_meta_attrs(context, resources, "target-role", "Stopped"):
             return False
-        if not utils.wait4dc("stop", not options.batch):
+        if not utils.wait_dc_stable("stop", not options.batch):
             return False
         logger.info("ordering %s to start", ", ".join(resources))
         return self._commit_meta_attrs(context, resources, "target-role", "Started")
@@ -339,7 +341,7 @@ class RscMgmt(command.UI):
         if not xmlutil.RscState().is_ms_or_promotable_clone(rsc):
             logger.error("%s is not a promotable resource", rsc)
             return False
-        role = utils.handle_role_for_ocf_1_1(constants.RSC_ROLE_PROMOTED_LEGACY)
+        role = utils.handle_role_for_ocf_1_1(constants.RSC_ROLE_PROMOTED)
         return utils.ext_cmd(self.rsc_setrole % (rsc, role)) == 0
 
     def do_scores(self, context):
@@ -368,7 +370,7 @@ class RscMgmt(command.UI):
         if not xmlutil.RscState().is_ms_or_promotable_clone(rsc):
             logger.error("%s is not a promotable resource", rsc)
             return False
-        role = utils.handle_role_for_ocf_1_1(constants.RSC_ROLE_UNPROMOTED_LEGACY)
+        role = utils.handle_role_for_ocf_1_1(constants.RSC_ROLE_UNPROMOTED)
         return utils.ext_cmd(self.rsc_setrole % (rsc, role)) == 0
 
     @command.completers(compl.resources)
@@ -491,69 +493,76 @@ class RscMgmt(command.UI):
         failcount <rsc> delete <node>
         failcount <rsc> show <node>"""
         def sec_to_ms(s):
-            return s + '000'
+            return '0' if s == '0' else s + '000'
 
         def ms_to_sec(m):
-            return m[:len(m)-3]
+            return '0' if m == '0' else str(int(m) // 1000)
+
+        USAGE = "Usage: failcount <rsc> set <node> <value> [operation] [interval]"
 
         if rsc not in compl.resources():
-            context.warning("Resource {} not exists in this cluster".format(rsc))
+            logger.warning("Resource %s not in this cluster", rsc)
             return
-        valid_cmd_list = ["set", "delete", "show"]
-        if cmd not in valid_cmd_list:
-            context.fatal_error("{} is not valid command(should be one of {})".format(cmd, valid_cmd_list))
+        if cmd not in ATTR_OPERATION_CMDS:
+            context.fatal_error(f"{cmd} is not valid command(should be one of {', '.join(ATTR_OPERATION_CMDS)})")
         nodeid = utils.get_nodeid_from_name(node)
         if nodeid is None:
-            context.fatal_error("Node {} not in this cluster".format(node))
+            context.fatal_error(f"Node {node} not in this cluster")
+        if value and (not value.isdigit() and value != "INFINITY"):
+            context.fatal_error("value should be a non-negative integer or 'INFINITY'")
 
         if cmd == "set":
             # query the current failcount status
-            query_cmd = "cibadmin -Ql --xpath '/cib/status/node_state[@id='{}']'".format(nodeid)
+            query_cmd = "cibadmin -Q --xpath '/cib/status/node_state[@id='{}']'".format(nodeid)
             rc, out, err = ShellUtils().get_stdout_stderr(query_cmd)
             if rc != 0:
                 context.fatal_error(err)
 
             # try to get failcount dict {operation:interval}
-            import re
-            failcount_res = re.findall(r'fail-count-{}#(.*)_([0-9]+)'.format(rsc), out)
-            if not failcount_res:
-                return True if value and int(value) == 0 else False
-            failcount_dict = dict(failcount_res)
-
-            # validate for operation and interval
-            if operation and operation not in failcount_dict.keys():
-                context.fatal_error("Usage: failcount <rsc> set <node> <value> [operation] [interval]\n\
-                           Should specify operation between \"{}\"".format(' '.join(failcount_dict.keys())))
-            if (operation and interval) and (operation, sec_to_ms(interval)) not in failcount_res:
-                context.fatal_error("Usage: failcount <rsc> set <node> <value> [operation] [interval]\n\
-                           Should specify (operation, interval) between {}".\
-                           format([(op, ms_to_sec(inter)) for op, inter in failcount_res]))
+            failcount_dict = dict(re.findall(rf'fail-count-{rsc}#(.*)_([0-9]+)', out))
+            if failcount_dict:
+                # validate for operation and interval
+                if operation and operation not in failcount_dict:
+                    context.fatal_error(f"Should specify operation between ({', '.join(failcount_dict.keys())})\n{USAGE}")
+                if (operation and interval) and (operation, sec_to_ms(interval)) not in failcount_dict.items():
+                    failcount_items = [(op, ms_to_sec(inter)) for op, inter in failcount_dict.items()]
+                    context.fatal_error(f"Should specify (operation, interval) between {failcount_items}\n{USAGE}")
+            else:
+                # no failcount entry for this resource and node
+                if operation and interval:
+                    rsc = f'{rsc}#{operation}_{sec_to_ms(interval)}'
+                else:
+                    # when no failcount entry, it's no harm to give a warning and return true
+                    # https://bugzilla.suse.com/show_bug.cgi?id=1180332#c11
+                    logger.warning("Should specify operation and interval\n%s", USAGE)
+                    return
 
             # just one failcount entry
-            if len(failcount_res) == 1:
-                operation = failcount_res[0][0]
-                interval = failcount_dict[operation]
-                rsc = '{}#{}_{}'.format(rsc, operation, interval)
+            if len(failcount_dict) == 1:
+                operation, interval_in_ms = failcount_dict.popitem()
+                rsc = f'{rsc}#{operation}_{interval_in_ms}'
 
             # multiple failcount entries for this resource and node
-            if len(failcount_res) > 1:
+            if len(failcount_dict) > 1:
                 if operation and interval:
-                    rsc = '{}#{}_{}'.format(rsc, operation, sec_to_ms(interval))
+                    rsc = f'{rsc}#{operation}_{sec_to_ms(interval)}'
                 elif int(value) == 0:
-                    # using '-P' option of 'crm_attribute' command
+                    op_interval_list = [
+                        f'{op}_{inter}'
+                        for op, inter in failcount_dict.items()
+                        if not operation or op == operation
+                    ]
+                    op_interval_str = '|'.join(op_interval_list)
+                    rsc = f'{rsc}#{op_interval_str}'
+                    # using '-P' option of 'crm_attribute' command, to match the pattern
                     cmd = "set_p"
-                    if operation:
-                        op_interval_str = '|'.join(['{}_{}'.format(operation, inter) for op, inter in failcount_res if op==operation])
-                    else:
-                        op_interval_str = '|'.join(['{}_{}'.format(op, inter) for op, inter in failcount_res])
-                    rsc = '{}#({})'.format(rsc, op_interval_str)
                 else:
                     # value != 0
-                    if operation and len([op for op, _ in failcount_res if op == operation]) == 1:
-                        rsc = '{}#{}_{}'.format(rsc, operation, failcount_dict[operation])
+                    if operation and len([op for op in failcount_dict if op == operation]) == 1:
+                        rsc = f'{rsc}#{operation}_{failcount_dict[operation]}'
                     else:
-                        context.fatal_error("Should specify (operation, interval) between {}".
-                                format([(op, ms_to_sec(inter)) for op, inter in failcount_res]))
+                        failcount_items = [(op, ms_to_sec(inter)) for op, inter in failcount_dict.items()]
+                        context.fatal_error(f"Should specify (operation, interval) between {failcount_items}\n{USAGE}")
 
         return ui_utils.manage_attr(context.get_command_name(), self.rsc_failcount,
                                     rsc, cmd, node, value)
@@ -699,19 +708,15 @@ class RscMgmt(command.UI):
         rsc = self._get_trace_rsc(rsc_id)
         if not rsc:
             return False
-
-        argl = list(args)
-        force = "force" in utils.fetch_opts(argl, ["force"]) or config.core.force
-        if len(argl) > 3:
+        if len(args) > 3:
             context.fatal_error(usage)
-        op=None
-        interval=None
-        dir=None
-        for arg in argl:
-            if arg[0] == '/':
-                if dir is not None:
+
+        op, interval, trace_dir = None, None, None
+        for arg in args:
+            if arg.startswith("/"):
+                if trace_dir is not None:
                     context.fatal_error(usage)
-                dir = arg
+                trace_dir = arg
             elif arg.isnumeric():
                 if interval is not None:
                     context.fatal_error(usage)
@@ -720,23 +725,24 @@ class RscMgmt(command.UI):
                 if op is not None:
                     context.fatal_error(usage)
                 op = arg
+        trace_dir = trace_dir or f"{config.path.heartbeat_dir}/trace_ra"
 
         if op == "probe":
             op = "monitor"
-            if interval is None:
-                interval = "0"
+            interval = interval or "0"
         if op is None:
-            self._trace_resource(context, rsc_id, rsc, dir)
+            self._trace_resource(context, rsc_id, rsc, trace_dir)
         elif interval is None:
-            self._trace_op(context, rsc_id, rsc, op, dir)
+            self._trace_op(context, rsc_id, rsc, op, trace_dir)
         else:
-            self._trace_op_interval(context, rsc_id, rsc, op, interval, dir)
+            self._trace_op_interval(context, rsc_id, rsc, op, interval, trace_dir)
+
         if not cib_factory.commit():
             return False
+
         rsc_type = rsc.node.get("type")
-        trace_dir = "{}/{}".format(dir, rsc_type) if dir else "{}/trace_ra/{}".format(config.path.heartbeat_dir, rsc_type)
-        logger.info("Trace for %s%s is written to %s", rsc_id, ":"+op if op else "", trace_dir)
-        if op is not None and op != "monitor":
+        logger.info("Trace for %s%s is written to %s", rsc_id, ":"+op if op else "", f"{trace_dir}/{rsc_type}")
+        if op and op != "monitor":
             logger.info("Trace set, restart %s to trace the %s operation", rsc_id, op)
         else:
             logger.info("Trace set, restart %s to trace non-monitor operations", rsc_id)
