@@ -2,6 +2,7 @@ import logging
 import os
 import socket
 import subprocess
+import time
 import typing
 
 from . import config
@@ -36,7 +37,7 @@ class UserOfHost:
         if cached is None:
             ret = self._get_user_of_host_from_config(host)
             if ret is None:
-                raise UserNotFoundError()
+                raise UserNotFoundError(f"UserNotFoundError: host={host}")
             else:
                 self._user_cache[host] = ret
                 return ret
@@ -48,11 +49,10 @@ class UserOfHost:
         if config.core.no_ssh:
             from .utils import NoSSHError
             raise NoSSHError(constants.NO_SSH_ERROR_MSG)
-
         local_user = None
         remote_user = None
         try:
-            local_user = 'root' if self.use_ssh_agent() else self.user_of(self.this_node())
+            local_user = self.user_of(self.this_node())
             remote_user = self.user_of(host)
             return local_user, remote_user
         except UserNotFoundError:
@@ -65,17 +65,13 @@ class UserOfHost:
                 else:
                     ret = self._guess_user_for_ssh(host)
                     if ret is None:
-                        raise UserNotFoundError from None
+                        raise UserNotFoundError(f'Passwordless ssh to host "{host}" does not work.') from None
                     else:
                         self._user_pair_cache[host] = ret
                         return ret
             else:
                 return cached
 
-    @staticmethod
-    def use_ssh_agent() -> bool:
-        return config.get_option('core', 'no_generating_ssh_key')
-
     @staticmethod
     def _get_user_of_host_from_config(host):
         try:
@@ -107,6 +103,7 @@ class UserOfHost:
             args.extend(['-o', 'BatchMode=yes', host, 'sudo', 'true'])
         else:
             args.extend(['-o', 'BatchMode=yes', host, 'true'])
+        logger.debug("subprocess.call(%s)", args)
         rc = subprocess.call(
             args,
             stdin=subprocess.DEVNULL,
@@ -118,6 +115,7 @@ class UserOfHost:
             user = userdir.getuser()
             return user, user
         else:
+            time.sleep(5)    # bsc#1243141: sshd PerSourcePenalties
             return None
 
 
