@@ -1,16 +1,17 @@
 import re
 from . import utils
 from .constants import SSH_OPTION
-from .bootstrap import invoke, invokerc, WATCHDOG_CFG, SYSCONFIG_SBD
 from .sh import ShellUtils
+from . import sbd
 
 
 class Watchdog(object):
     """
     Class to find valid watchdog device name
     """
+    WATCHDOG_CFG = "/etc/modules-load.d/watchdog.conf"
     QUERY_CMD = "sudo sbd query-watchdog"
-    DEVICE_FIND_REGREX = "\[[0-9]+\] (/dev/.*)\n.*\nDriver: (.*)"
+    DEVICE_FIND_REGREX = "\\[[0-9]+\\] (/dev/.*)\n.*\nDriver: (.*)"
 
     def __init__(self, _input=None, remote_user=None, peer_host=None):
         """
@@ -27,7 +28,7 @@ class Watchdog(object):
         return self._watchdog_device_name
 
     @staticmethod
-    def _verify_watchdog_device(dev, ignore_error=False):
+    def verify_watchdog_device(dev, ignore_error=False):
         """
         Use wdctl to verify watchdog device
         """
@@ -44,15 +45,15 @@ class Watchdog(object):
         """
         Load specific watchdog driver
         """
-        invoke("echo {} > {}".format(driver, WATCHDOG_CFG))
-        invoke("systemctl restart systemd-modules-load")
+        ShellUtils().get_stdout_stderr(f"echo {driver} > {Watchdog.WATCHDOG_CFG}")
+        ShellUtils().get_stdout_stderr("systemctl restart systemd-modules-load")
 
     @staticmethod
-    def _get_watchdog_device_from_sbd_config():
+    def get_watchdog_device_from_sbd_config():
         """
         Try to get watchdog device name from sbd config file
         """
-        conf = utils.parse_sysconfig(SYSCONFIG_SBD)
+        conf = utils.parse_sysconfig(sbd.SBDManager.SYSCONFIG_SBD)
         return conf.get("SBD_WATCHDOG_DEV")
 
     @staticmethod
@@ -61,7 +62,7 @@ class Watchdog(object):
         Check if driver was already loaded
         """
         _, out, _ = ShellUtils().get_stdout_stderr("lsmod")
-        return re.search("\n{}\s+".format(driver), out)
+        return re.search("\n{}\\s+".format(driver), out)
 
     def _set_watchdog_info(self):
         """
@@ -81,7 +82,7 @@ class Watchdog(object):
         Get watchdog device name which has driver_name
         """
         for device, driver in self._watchdog_info_dict.items():
-            if driver == driver_name and self._verify_watchdog_device(device):
+            if driver == driver_name and self.verify_watchdog_device(device):
                 return device
         return None
 
@@ -108,7 +109,7 @@ class Watchdog(object):
         Get first unused watchdog device name
         """
         for dev in self._watchdog_info_dict:
-            if self._verify_watchdog_device(dev, ignore_error=True):
+            if self.verify_watchdog_device(dev, ignore_error=True):
                 return dev
         return None
 
@@ -120,8 +121,8 @@ class Watchdog(object):
           3. Set the self._input as softdog
         """
         if not self._input:
-            dev = self._get_watchdog_device_from_sbd_config()
-            if dev and self._verify_watchdog_device(dev, ignore_error=True):
+            dev = self.get_watchdog_device_from_sbd_config()
+            if dev and self.verify_watchdog_device(dev, ignore_error=True):
                 self._input = dev
                 return
             first_unused = self._get_first_unused_device()
@@ -131,7 +132,7 @@ class Watchdog(object):
         """
         Is an unused watchdog device
         """
-        if dev in self._watchdog_info_dict and self._verify_watchdog_device(dev):
+        if dev in self._watchdog_info_dict and self.verify_watchdog_device(dev):
             return True
         return False
 
@@ -142,9 +143,9 @@ class Watchdog(object):
         """
         self._set_watchdog_info()
 
-        res = self._get_watchdog_device_from_sbd_config()
+        res = self.get_watchdog_device_from_sbd_config()
         if not res:
-            utils.fatal("Failed to get watchdog device from {}".format(SYSCONFIG_SBD))
+            utils.fatal("Failed to get watchdog device from {}".format(sbd.SBDManager.SYSCONFIG_SBD))
         self._input = res
 
         if not self._valid_device(self._input):
@@ -164,7 +165,8 @@ class Watchdog(object):
             return
 
         # self._input is invalid, exit
-        if not invokerc("modinfo {}".format(self._input)):
+        rc, _, _ = ShellUtils().get_stdout_stderr(f"modinfo {self._input}")
+        if rc != 0:
             utils.fatal("Should provide valid watchdog device or driver name by -w option")
 
         # self._input is a driver name, load it if it was unloaded
@@ -177,3 +179,9 @@ class Watchdog(object):
         if res:
             self._watchdog_device_name = res
             return
+
+    @classmethod
+    def get_watchdog_device(cls, dev_or_driver=None):
+        w = cls(_input=dev_or_driver)
+        w.init_watchdog()
+        return w.watchdog_device_name
