@@ -1,3 +1,4 @@
+import ast
 import re
 import time
 import os
@@ -7,8 +8,10 @@ import yaml
 import behave
 from behave import given, when, then
 import behave_agent
-from crmsh import corosync, sbd, userdir, bootstrap
+from crmsh import corosync, userdir, bootstrap
 from crmsh import utils as crmutils
+from crmsh import sbd
+from crmsh import ui_configure
 from crmsh.sh import ShellUtils
 from utils import check_cluster_state, check_service_state, online, run_command, me, \
                   run_command_local_or_remote, file_in_archive, \
@@ -17,8 +20,8 @@ import const
 
 
 def _parse_str(text):
-    return text[1:-1].encode('utf-8').decode('unicode_escape')
-_parse_str.pattern='".*"'
+    return ast.literal_eval(text)
+_parse_str.pattern='"([^"]|\\")*?"'
 
 
 behave.use_step_matcher("cfparse")
@@ -169,6 +172,13 @@ def step_impl(context, msg):
     context.stderr = None
 
 
+@then('Expect stdout contains snippets [{snippets:str+}].')
+def step_impl(context, snippets):
+    for snippet in snippets:
+        assert_in(snippet, context.stdout)
+    context.stdout = None
+
+
 @then('Expected regex "{reg_str}" in stdout')
 def step_impl(context, reg_str):
     res = re.search(reg_str, context.stdout)
@@ -238,6 +248,13 @@ def step_impl(context, nodelist):
     assert online(context, nodelist) is True
 
 
+@then('Remote online nodes are "{nodelist}"')
+def step_impl(context, nodelist):
+    _, out, _ = run_command(context, 'crm_mon -1|grep RemoteOnline:')
+    for node in nodelist.split():
+        assert node in out, "Node {} not found in {}".format(node, out)
+
+
 @then('Node "{node}" is standby')
 def step_impl(context, node):
     assert crmutils.is_standby(node) is True
@@ -250,9 +267,9 @@ def step_impl(context, node):
 
 @then('IP "{addr}" is used by corosync on "{node}"')
 def step_impl(context, addr, node):
-    _, out, _ = run_command_local_or_remote(context, 'corosync-cfgtool -s', node)
-    res = re.search(r' {}\n'.format(addr), out)
-    assert bool(res) is True
+    nodeid = crmutils.get_nodeid_from_name(node)
+    _, out, _ = run_command_local_or_remote(context, f'corosync-cfgtool -a {nodeid}', node)
+    assert addr in out.split()
 
 
 @then('Cluster name is "{name}"')
@@ -268,9 +285,16 @@ def step_impl(context, addr):
     assert bool(res) is True
 
 
-@then('Cluster is using udpu transport mode')
-def step_impl(context):
-    assert corosync.get_value('totem.transport') == 'udpu'
+@then('Cluster is using "{transport_type}" transport mode')
+def step_impl(context, transport_type):
+    assert corosync.get_value('totem.transport') == transport_type
+    _, out, _ = run_command(context, 'corosync-cfgtool -s')
+    assert re.search(f'transport {transport_type}\n', out) is not None
+
+
+@then('two_node in corosync.conf is "{number}"')
+def step_impl(context, number):
+    assert corosync.get_value('quorum.two_node') == number
 
 
 @then('Show cluster status on "{addr}"')
@@ -432,7 +456,7 @@ def step_impl(context, res_id, node):
 
 @then('SBD option "{key}" value is "{value}"')
 def step_impl(context, key, value):
-    res = sbd.SBDManager.get_sbd_value_from_config(key)
+    res = sbd.SBDUtils.get_sbd_value_from_config(key)
     assert_eq(value, res)
 
 
@@ -446,27 +470,27 @@ def step_impl(context, key, dev, value):
 def step_impl(context, key, value):
     res = crmutils.get_property(key)
     assert res is not None
-    assert_eq(value, str(res))
+    assert_eq(value.strip('s'), str(res).strip('s'))
 
 
 @then('Property "{key}" in "{type}" is "{value}"')
 def step_impl(context, key, type, value):
     res = crmutils.get_property(key, type)
     assert res is not None
-    assert_eq(value, str(res))
+    assert_eq(value.strip('s'), str(res).strip('s'))
 
 
 @then('Parameter "{param_name}" not configured in "{res_id}"')
 def step_impl(context, param_name, res_id):
     _, out, _ = run_command(context, "crm configure show {}".format(res_id))
-    result = re.search("params {}=".format(param_name), out)
+    result = re.search("params .*{}=".format(param_name), out)
     assert result is None
 
 
 @then('Parameter "{param_name}" configured in "{res_id}"')
 def step_impl(context, param_name, res_id):
     _, out, _ = run_command(context, "crm configure show {}".format(res_id))
-    result = re.search("params {}=".format(param_name), out)
+    result = re.search("params .*{}=".format(param_name), out)
     assert result is not None
 
 
@@ -537,8 +561,9 @@ def step_impl(context, count, node):
         index += 1
     return False
 
-@then('Check passwordless for hacluster between "{nodelist}"')
-def step_impl(context, nodelist):
+@then('Check passwordless for hacluster between "{nodelist}" "{result}"')
+def step_impl(context, nodelist, result):
+    assert result in ("successfully", "failed")
     if userdir.getuser() != 'root' or userdir.get_sudoer():
         return True
     failed = False
@@ -553,7 +578,10 @@ def step_impl(context, nodelist):
             if rc != 0:
                 failed = True
                 context.logger.error(f"There is no passwordless configured from {nodes[i]} to {nodes[j]} under 'hacluster'")
-    assert not failed
+    if result == "successfully":
+        assert not failed
+    else:
+        assert failed
 
 
 @then('Check user shell for hacluster between "{nodelist}"')
@@ -597,3 +625,87 @@ def step_impl(context, nodes):
 EOF''',
             user='root',
         )
+
+
+@when('Set to previous schema version')
+def step_impl(context):
+    _, current_schema, _ = ShellUtils().get_stdout_stderr("crm configure schema")
+    assert current_schema
+    schema_list = ui_configure.schema_completer(None)
+    assert schema_list
+    assert current_schema in schema_list
+    previous_schema = schema_list[schema_list.index(current_schema) - 1]
+    context.previous_schema = previous_schema
+    rc, _, _ = ShellUtils().get_stdout_stderr(f"crm configure schema {previous_schema}")
+    assert rc == 0
+
+
+@then('The schema version is the previous')
+def step_impl(context):
+    rc, schema, _ = ShellUtils().get_stdout_stderr("crm configure schema")
+    assert rc == 0
+    assert schema == context.previous_schema
+
+
+@given('Get the latest schema version')
+def step_impl(context):
+    schema_list = ui_configure.schema_completer(None)
+    assert schema_list
+    context.schema_latest = schema_list[-1]
+
+
+@when('Use crm configure upgrade to upgrade the schema')
+def step_impl(context):
+    rc, _, _ = ShellUtils().get_stdout_stderr("crm configure upgrade force")
+    assert rc == 0
+
+
+@then('The schema version is the latest')
+def step_impl(context):
+    rc, schema, _ = ShellUtils().get_stdout_stderr("crm configure schema")
+    assert rc == 0
+    assert schema == context.schema_latest
+
+
+def firewall_cmd(context, node):
+    rc, _, _ = run_command_local_or_remote(context, "firewall-cmd --state", node, exit_on_fail=False)
+    return "firewall-cmd" if rc == 0 else "firewall-offline-cmd"
+
+
+@then('Port "{port}" protocol "{proto}" is opened on "{node}"')
+def step_impl(context, port, proto, node):
+    cmd = f"{firewall_cmd(context, node)} --list-ports"
+    rc, out, _ = run_command_local_or_remote(context, cmd, node, exit_on_fail=False)
+    assert rc == 0
+    assert f"{port}/{proto}" in out.split()
+
+
+@given('The "{service}" firewalld service is available on "{node}"')
+def step_impl(context, service, node):
+    cmd = f"{firewall_cmd(context, node)} --info-service={service}"
+    rc, _, _ = run_command_local_or_remote(context, cmd, node, exit_on_fail=False)
+    assert rc == 0
+
+
+def firewalld_list_services(context, node):
+    cmd = f"{firewall_cmd(context, node)} --list-services"
+    _, out, _ = run_command_local_or_remote(context, cmd, node, exit_on_fail=False)
+    return out or ""
+
+
+@then('The "{service}" firewalld service is added on "{node}"')
+def step_impl(context, service, node):
+    services = firewalld_list_services(context, node)
+    assert service in services.split()
+
+
+@then('The "{service}" firewalld service is not added on "{node}"')
+def step_impl(context, service, node):
+    services = firewalld_list_services(context, node)
+    assert service not in services.split()
+
+
+@given('The "{service}" firewalld service is not added on "{node}"')
+def step_impl(context, service, node):
+    services = firewalld_list_services(context, node)
+    assert service not in services.split()
