@@ -9,7 +9,7 @@ except ImportError:
 
 from crmsh import cibconfig
 from lxml import etree
-from crmsh import xmlutil
+from crmsh import xmlutil, utils
 
 factory = cibconfig.cib_factory
 
@@ -472,7 +472,7 @@ def test_group_constraint_location():
     factory.create_object('group', 'g1', 'p1', 'p2')
     factory.create_object('location', 'loc-p1', 'p1', 'inf:', 'node1')
     c = factory.find_object('loc-p1')
-    assert c and c.check_sanity() == 0
+    assert c and c.check_sanity() == utils.VerifyResult.SUCCESS
 
 
 def test_group_constraint_colocation():
@@ -484,7 +484,8 @@ def test_group_constraint_colocation():
     factory.create_object('group', 'g1', 'p1', 'p2')
     factory.create_object('colocation', 'coloc-p1-p2', 'inf:', 'p1', 'p2')
     c = factory.find_object('coloc-p1-p2')
-    assert c and c.check_sanity() > 0
+    rc = c.check_sanity()
+    assert c and bool(rc) is True and utils.VerifyResult.WARNING in rc
 
 
 def test_group_constraint_colocation_rscset():
@@ -497,7 +498,8 @@ def test_group_constraint_colocation_rscset():
     factory.create_object('group', 'g1', 'p1', 'p2')
     factory.create_object('colocation', 'coloc-p1-p2-p3', 'inf:', 'p1', 'p2', 'p3')
     c = factory.find_object('coloc-p1-p2-p3')
-    assert c and c.check_sanity() > 0
+    rc = c.check_sanity()
+    assert c and bool(rc) is True and utils.VerifyResult.WARNING in rc
 
 
 def test_clone_constraint_colocation_rscset():
@@ -510,7 +512,8 @@ def test_clone_constraint_colocation_rscset():
     factory.create_object('clone', 'c1', 'p1')
     factory.create_object('colocation', 'coloc-p1-p2-p3', 'inf:', 'p1', 'p2', 'p3')
     c = factory.find_object('coloc-p1-p2-p3')
-    assert c and c.check_sanity() > 0
+    rc = c.check_sanity()
+    assert c and bool(rc) is True and utils.VerifyResult.WARNING in rc
 
 
 def test_existing_node_resource():
@@ -572,9 +575,7 @@ primitive p1 ocf:pacemaker:Dummy
 primitive p2 ocf:heartbeat:Delay \
     params startdelay=2 mondelay=2 stopdelay=2
 primitive p3 ocf:pacemaker:Dummy
-primitive st stonith:null params hostlist=node1
 clone c1 p1
-ms m1 p2
 op_defaults timeout=60s
 """)
     assert ok
@@ -584,10 +585,6 @@ op_defaults timeout=60s
     ok = obj.save("""op_defaults timeout=2m
 node node1 \
     attributes mem=16G
-primitive st stonith:null \
-    params hostlist='node1' \
-    meta description="some description here" requires=nothing \
-    op monitor interval=60m
 primitive p1 ocf:heartbeat:Dummy \
     op monitor interval=60m \
     op monitor interval=120m OCF_CHECK_LEVEL=10
@@ -703,8 +700,6 @@ primitive d0 Dummy \
 primitive d1 Dummy
 primitive d2 Dummy
 # Never use this STONITH agent in production!
-primitive development-stonith stonith:null \
-	params hostlist="webui node1 node2 node3"
 primitive proxy systemd:haproxy \
 	op monitor interval=10s
 primitive proxy-vip IPaddr2 \
@@ -778,8 +773,8 @@ def test_bug_110():
     """
     configuring attribute-based fencing-topology
     """
-    factory.create_object(*"primitive stonith-libvirt stonith:null".split())
-    factory.create_object(*"primitive fence-nova stonith:null".split())
+    factory.create_object(*"primitive stonith-libvirt stonith:fence_sbd".split())
+    factory.create_object(*"primitive fence-nova stonith:fence_sbd".split())
     cmd = "fencing_topology attr:OpenStack-role=compute stonith-libvirt,fence-nova".split()
     ok = factory.create_object(*cmd)
     assert ok
@@ -788,7 +783,7 @@ def test_bug_110():
 
     for o in obj.obj_set:
         if o.node.tag == 'fencing-topology':
-            assert o.check_sanity() == 0
+            assert o.check_sanity() == utils.VerifyResult.SUCCESS
 
 
 @mock.patch("crmsh.log.LoggerUtils.line_number")
