@@ -124,7 +124,7 @@ Expected State:    a) sbd process restarted
     @mock.patch('crmsh.crash_test.task.TaskKill.process_monitor')
     @mock.patch('crmsh.crash_test.task.Task.fence_action_monitor')
     @mock.patch('threading.Thread')
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.Task.info')
     @mock.patch('crmsh.crash_test.utils.get_process_status')
     def test_run(self, mock_status, mock_info, mock_run, mock_thread, mock_fence_monitor, mock_process_monitor):
@@ -203,14 +203,14 @@ class TestTaskCheck(TestCase):
         """
 
     @mock.patch('crmsh.crash_test.utils.msg_info')
-    @mock.patch('crmsh.crash_test.utils.now')
-    def setUp(self, mock_now, mock_msg_info):
+    def setUp(self, mock_msg_info):
         """
         Test setUp.
         """
-        mock_now.return_value = "2019/07/10 01:15:15"
+        fake_timedate = datetime(2019, 7, 10, 1, 15, 15)
         main.ctx = mock.Mock(task_list=[{"process_name": "xin", "age": 38}])
         self.task_check_inst = task.TaskCheck("task check job1", quiet=False)
+        self.task_check_inst.timestamp = fake_timedate
         self.task_check_inst_quiet = task.TaskCheck("task check job1", quiet=True)
 
     def tearDown(self):
@@ -343,7 +343,7 @@ Fence timeout:     60
         mock_result.assert_called_once_with()
         mock_json.assert_called_once_with()
 
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.Task.task_pre_check')
     def test_pre_check_no_cmd(self, mock_pre_check, mock_run):
         mock_run.return_value = (1, None, "error")
@@ -354,7 +354,7 @@ Fence timeout:     60
         mock_pre_check.assert_called_once_with()
 
     @mock.patch('crmsh.crash_test.utils.online_nodes')
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.Task.task_pre_check')
     def test_pre_check_error(self, mock_pre_check, mock_run, mock_online_nodes):
         mock_run.return_value = (0, None, None)
@@ -365,7 +365,7 @@ Fence timeout:     60
         mock_run.assert_called_once_with("which iptables")
         mock_online_nodes.assert_called_once_with()
 
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.crmshutils.get_iplist_from_name')
     @mock.patch('crmsh.crash_test.task.Task.info')
     @mock.patch('crmsh.crash_test.utils.peer_node_list')
@@ -394,7 +394,7 @@ Fence timeout:     60
         self.task_sp_inst.un_block()
         mock_unblock_iptables.assert_called_once_with()
 
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.crmshutils.get_iplist_from_name')
     @mock.patch('crmsh.crash_test.task.Task.info')
     def test_un_block_iptables(self, mock_info, mock_get_iplist, mock_run):
@@ -480,7 +480,7 @@ Fence timeout:     60
         mock_result.assert_called_once_with()
         mock_json.assert_called_once_with()
 
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.Task.task_pre_check')
     def test_pre_check_no_cmd(self, mock_pre_check, mock_run):
         mock_run.return_value = (1, None, "error")
@@ -491,7 +491,7 @@ Fence timeout:     60
         mock_pre_check.assert_called_once_with()
 
     @mock.patch('crmsh.crash_test.utils.check_node_status')
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.Task.task_pre_check')
     def test_pre_check_error(self, mock_pre_check, mock_run, mock_node_status):
         mock_run.side_effect = [(0, None, None), (0, None, None), (0, None, None)]
@@ -508,7 +508,7 @@ Fence timeout:     60
 
     @mock.patch('crmsh.crash_test.task.Task.fence_action_monitor')
     @mock.patch('threading.Thread')
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
+    @mock.patch('crmsh.sh.ShellUtils.get_stdout_stderr')
     @mock.patch('crmsh.crash_test.task.Task.info')
     def test_run(self, mock_info, mock_run, mock_thread, mock_monitor):
         mock_thread_inst = mock.Mock()
@@ -559,15 +559,14 @@ class TestTask(TestCase):
         """
 
     @mock.patch('crmsh.crash_test.utils.msg_info')
-    @mock.patch('crmsh.crash_test.utils.now')
-    def setUp(self, mock_now, mock_info):
+    def setUp(self, mock_info):
         """
         Test setUp.
         """
-        mock_now.return_value = "2019/07/10 01:15:15"
+        fake_now = datetime(2019, 7, 10, 1, 15, 15)
         main.ctx = mock.Mock(task_list={"process_name": "xin", "age": 38})
         self.task_inst = task.Task("task description", flush=True)
-        mock_now.assert_called_once_with()
+        self.task_inst.timestamp = fake_now
 
     def tearDown(self):
         """
@@ -589,7 +588,7 @@ class TestTask(TestCase):
     def test_to_json(self):
         self.task_inst.to_json()
 
-    @mock.patch('crmsh.crash_test.task.crmshutils.service_is_active')
+    @mock.patch('crmsh.service_manager.ServiceManager.service_is_active')
     def test_task_pre_check_exception(self, mock_active):
         mock_active.return_value = False
         with self.assertRaises(task.TaskError) as err:
@@ -597,7 +596,7 @@ class TestTask(TestCase):
         self.assertEqual("Cluster not running!", str(err.exception))
         mock_active.assert_called_once_with("pacemaker.service")
 
-    @mock.patch('crmsh.crash_test.task.crmshutils.service_is_active')
+    @mock.patch('crmsh.service_manager.ServiceManager.service_is_active')
     def test_task_pre_check_exception_no_fence(self, mock_active):
         mock_active.return_value = True
         self.task_inst.get_fence_info = mock.Mock()
@@ -647,7 +646,7 @@ class TestTask(TestCase):
     def test_build_base_result(self):
         self.task_inst.build_base_result()
         expected_result = {
-            "Timestamp": self.task_inst.timestamp,
+            "Timestamp": '2019/07/10 01:15:15',
             "Description": self.task_inst.description,
             "Messages": []
         }
@@ -666,45 +665,33 @@ class TestTask(TestCase):
         mock_ask.assert_called_once_with(task.Task.REBOOT_WARNING)
         self.task_inst.info.assert_called_once_with("Testcase cancelled")
 
-    @mock.patch('crmsh.crash_test.utils.str_to_datetime')
     @mock.patch('time.sleep')
     @mock.patch('crmsh.crash_test.task.Task.info')
-    @mock.patch('crmsh.crash_test.task.crmshutils.get_stdout_stderr')
-    def test_fence_action_monitor(self, mock_run, mock_info, mock_sleep, mock_datetime):
+    @mock.patch('crmsh.xmlutil.CrmMonXmlParser')
+    def test_fence_action_monitor(self, mock_parser, mock_info, mock_sleep):
         self.task_inst.thread_stop_event = mock.Mock()
-        self.task_inst.thread_stop_event.is_set.side_effect = [False, False, False, False]
+        self.task_inst.thread_stop_event.is_set.side_effect = [False, False]
         self.task_inst.fence_start_event = mock.Mock()
+        self.task_inst.fence_start_event.is_set.side_effect = [False, True]
         self.task_inst.fence_finish_event = mock.Mock()
-        output = "Pending Fencing Actions:\n  * reboot of 15sp2-2 pending: client=pacemaker-controld.2430, origin=15sp2-1"
-        output2 = "Node 15sp2-2 last fenced at: Tue Jan 19 16:08:37 2021"
-        mock_run.side_effect = [(1, None, None), (0, output, None), (1, None, None), (0, output2, None)]
-        self.task_inst.timestamp = "2021/01/19 16:08:24"
-        mock_datetime.side_effect = [
-            datetime.strptime(self.task_inst.timestamp, '%Y/%m/%d %H:%M:%S'),
-            datetime.strptime("Tue Jan 19 16:08:37 2021", '%a %b %d %H:%M:%S %Y')
+        mock_parser_inst = mock.Mock()
+        mock_parser.return_value = mock_parser_inst
+        mock_parser_inst.get_last_fence_event_info.side_effect = [
+            {"target": "node2", "origin": "node1", "status": "pending", "completed": ""},
+            {"target": "node2", "origin": "node1", "status": "success", "completed": "2025-05-30 10:41:58.376958 +08:00"},
         ]
 
         self.task_inst.fence_action_monitor()
 
-        self.task_inst.thread_stop_event.is_set.assert_has_calls([
-            mock.call(),
-            mock.call(),
-            mock.call(),
-            mock.call()
-            ])
-        mock_run.assert_has_calls([
-            mock.call("crm_mon -1|grep -A1 \"Fencing Actions:\""),
-            mock.call("crm_mon -1|grep -A1 \"Fencing Actions:\""),
-            mock.call(config.FENCE_HISTORY.format(node="15sp2-2")),
-            mock.call(config.FENCE_HISTORY.format(node="15sp2-2"))
-            ])
+        self.task_inst.thread_stop_event.is_set.assert_has_calls([mock.call(), mock.call()])
         mock_info.assert_has_calls([
-            mock.call("Node \"15sp2-2\" will be fenced by \"15sp2-1\"!"),
-            mock.call("Node \"15sp2-2\" was successfully fenced by \"15sp2-1\"")
+            mock.call("Node \"node2\" will be fenced by \"node1\"!"),
+            mock.call("Node \"node2\" was fenced by \"node1\" at 2025-05-30 10:41:58.376958 +08:00")
             ])
         self.task_inst.fence_start_event.set.assert_called_once_with()
         self.task_inst.fence_finish_event.set.assert_called_once_with()
 
+
 class TestFixSBD(TestCase):
     """
     Class to test TaskFixSBD of task.py
