@@ -4,58 +4,57 @@ from __future__ import unicode_literals
 
 
 from crmsh import handles
-from nose.tools import eq_
 
 
 def test_basic():
     t = """{{foo}}"""
-    eq_("hello", handles.parse(t, {'foo': 'hello'}))
+    assert "hello" == handles.parse(t, {'foo': 'hello'})
     t = """{{foo:bar}}"""
-    eq_("hello", handles.parse(t, {'foo': {'bar': 'hello'}}))
+    assert "hello" == handles.parse(t, {'foo': {'bar': 'hello'}})
     t = """{{wiz}}"""
-    eq_("", handles.parse(t, {'foo': {'bar': 'hello'}}))
+    assert "" == handles.parse(t, {'foo': {'bar': 'hello'}})
     t = """{{foo}}.{{wiz}}"""
-    eq_("a.b", handles.parse(t, {'foo': "a", 'wiz': "b"}))
+    assert "a.b" == handles.parse(t, {'foo': "a", 'wiz': "b"})
     t = """Here's a line of text
     followed by another line
     followed by some {{foo}}.{{wiz}}
     and then some at the end"""
-    eq_("""Here's a line of text
+    assert """Here's a line of text
     followed by another line
     followed by some a.b
-    and then some at the end""", handles.parse(t, {'foo': "a", 'wiz': "b"}))
+    and then some at the end""" == handles.parse(t, {'foo': "a", 'wiz': "b"})
 
 
 def test_weird_chars():
     t = "{{foo#_bar}}"
-    eq_("hello", handles.parse(t, {'foo#_bar': 'hello'}))
+    assert "hello" == handles.parse(t, {'foo#_bar': 'hello'})
     t = "{{_foo$bar_}}"
-    eq_("hello", handles.parse(t, {'_foo$bar_': 'hello'}))
+    assert "hello" == handles.parse(t, {'_foo$bar_': 'hello'})
 
 
 def test_conditional():
     t = """{{#foo}}before{{foo:bar}}after{{/foo}}"""
-    eq_("beforehelloafter", handles.parse(t, {'foo': {'bar': 'hello'}}))
-    eq_("", handles.parse(t, {'faa': {'bar': 'hello'}}))
+    assert "beforehelloafter" == handles.parse(t, {'foo': {'bar': 'hello'}})
+    assert "" == handles.parse(t, {'faa': {'bar': 'hello'}})
 
     t = """{{#cond}}before{{foo:bar}}after{{/cond}}"""
-    eq_("beforehelloafter", handles.parse(t, {'foo': {'bar': 'hello'}, 'cond': True}))
-    eq_("", handles.parse(t, {'foo': {'bar': 'hello'}, 'cond': False}))
+    assert "beforehelloafter" == handles.parse(t, {'foo': {'bar': 'hello'}, 'cond': True})
+    assert "" == handles.parse(t, {'foo': {'bar': 'hello'}, 'cond': False})
 
 
 def test_iteration():
     t = """{{#foo}}!{{foo:bar}}!{{/foo}}"""
-    eq_("!hello!!there!", handles.parse(t, {'foo': [{'bar': 'hello'}, {'bar': 'there'}]}))
+    assert "!hello!!there!" == handles.parse(t, {'foo': [{'bar': 'hello'}, {'bar': 'there'}]})
 
 
 def test_result():
     t = """{{obj}}
     group g1 {{obj:id}}
 """
-    eq_("""primitive d0 Dummy
+    assert """primitive d0 Dummy
     group g1 d0
-""", handles.parse(t, {'obj': handles.value({'id': 'd0'}, 'primitive d0 Dummy')}))
-    eq_("\n    group g1 \n", handles.parse(t, {}))
+""" == handles.parse(t, {'obj': handles.value({'id': 'd0'}, 'primitive d0 Dummy')})
+    assert "\n    group g1 \n" == handles.parse(t, {})
 
 
 def test_result2():
@@ -65,11 +64,11 @@ def test_result2():
 {{obj}}
 {{/obj}}
 """
-    eq_("""primitive d0 Dummy
+    assert """primitive d0 Dummy
     group g1 d0
 primitive d0 Dummy
-""", handles.parse(t, {'obj': handles.value({'id': 'd0'}, 'primitive d0 Dummy')}))
-    eq_("\n    group g1 \n", handles.parse(t, {}))
+""" == handles.parse(t, {'obj': handles.value({'id': 'd0'}, 'primitive d0 Dummy')})
+    assert "\n    group g1 \n" == handles.parse(t, {})
 
 
 def test_mustasche():
@@ -86,10 +85,10 @@ Well, {{taxed_value}} dollars, after taxes.
         "in_ca": True
     }
 
-    eq_("""Hello Chris
+    assert """Hello Chris
 You have just won 10000 dollars!
 Well, 6000.0 dollars, after taxes.
-""", handles.parse(t, v))
+""" == handles.parse(t, v)
 
 
 def test_invert():
@@ -104,9 +103,9 @@ No repos :(
         "repo": []
     }
 
-    eq_("""
+    assert """
 No repos :(
-""", handles.parse(t, v))
+""" == handles.parse(t, v)
 
 
 def test_invert_2():
@@ -123,10 +122,10 @@ bar
         "repo": []
     }
 
-    eq_("""foo
+    assert """foo
 No repos :(
 bar
-""", handles.parse(t, v))
+""" == handles.parse(t, v)
 
 
 def test_cib():
@@ -164,4 +163,4 @@ colocation nfs-with-rootfs inf: g-nfs c-rfs
         'virtual-ip': handles.value({'id': 'vip'},
                                     'primitive vip IPaddr2\n  params ip=192.168.0.2'),
     }
-    eq_(r, handles.parse(t, v))
+    assert r == handles.parse(t, v)
