@@ -4,7 +4,6 @@ from __future__ import unicode_literals
 
 
 from crmsh import cibconfig
-from nose.tools import eq_, with_setup
 
 factory = cibconfig.cib_factory
 
@@ -15,27 +14,25 @@ def assert_in(needle, haystack):
         raise AssertionError(message)
 
 
-def setup_func():
+def setup_function():
     "set up test fixtures"
     from crmsh import idmgmt
     idmgmt.clear()
 
 
-def teardown_func():
+def teardown_function():
     pass
 
 
-@with_setup(setup_func, teardown_func)
 def test_nodes_nocli():
     for n in factory.node_id_list():
         obj = factory.find_object(n)
         if obj is not None:
             assert obj.node is not None
-            eq_(True, obj.cli_use_validate())
-            eq_(False, obj.nocli)
+            assert True == obj.cli_use_validate()
+            assert False == obj.nocli
 
 
-@with_setup(setup_func, teardown_func)
 def test_show():
     setobj = cibconfig.mkset_obj()
     s = setobj.repr_nopretty()
