@@ -1,4 +1,3 @@
-import os
 import unittest
 import socket
 
@@ -12,10 +11,6 @@ from crmsh import sbd
 from crmsh import qdevice, lock
 
 
-F2 = open(os.path.join(os.path.dirname(__file__), 'corosync.conf.2')).read()
-F4 = open(os.path.join(os.path.dirname(__file__), 'corosync.conf.3')).read()
-
-
 @mock.patch('crmsh.utils.calculate_quorate_status')
 @mock.patch('crmsh.utils.get_quorum_votes_dict')
 def test_evaluate_qdevice_quorum_effect_restart(mock_get_dict, mock_quorate):
@@ -44,7 +39,7 @@ def test_evaluate_qdevice_quorum_effect_reload(mock_get_dict, mock_quorate):
 def test_evaluate_qdevice_quorum_effect_later(mock_get_dict, mock_quorate, mock_parser):
     mock_get_dict.return_value = {'Expected': '2', 'Total': '2'}
     mock_quorate.return_value = False
-    mock_parser().is_any_resource_running.return_value = True
+    mock_parser().is_non_stonith_resource_running.return_value = True
     res = qdevice.evaluate_qdevice_quorum_effect(qdevice.QDEVICE_REMOVE)
     assert res == qdevice.QdevicePolicy.QDEVICE_RESTART_LATER
     mock_get_dict.assert_called_once_with()
@@ -57,7 +52,7 @@ def test_evaluate_qdevice_quorum_effect_later(mock_get_dict, mock_quorate, mock_
 def test_evaluate_qdevice_quorum_effect(mock_get_dict, mock_quorate, mock_parser):
     mock_get_dict.return_value = {'Expected': '2', 'Total': '2'}
     mock_quorate.return_value = False
-    mock_parser().is_any_resource_running.return_value = False
+    mock_parser().is_non_stonith_resource_running.return_value = False
     res = qdevice.evaluate_qdevice_quorum_effect(qdevice.QDEVICE_REMOVE)
     assert res == qdevice.QdevicePolicy.QDEVICE_RESTART
     mock_get_dict.assert_called_once_with()
@@ -742,66 +737,6 @@ class TestQDevice(unittest.TestCase):
         mock_fetch_p12_from_cluster.assert_called_once_with()
         mock_import_p12_on_local.assert_called_once_with()
 
-    @mock.patch("crmsh.utils.str2file")
-    @mock.patch("crmsh.corosync.make_section")
-    @mock.patch("crmsh.corosync.Parser")
-    @mock.patch("crmsh.corosync.conf")
-    @mock.patch("crmsh.utils.read_from_file")
-    def test_write_qdevice_config(self, mock_read_file, mock_conf, mock_parser, mock_mksection, mock_str2file):
-        mock_mksection.side_effect = [
-            ["device {", "}"],
-            ["net {", "}"]
-        ]
-        mock_read_file.return_value = "data"
-        mock_conf.side_effect = ["corosync.conf", "corosync.conf"]
-        mock_instance = mock.Mock()
-        mock_parser.return_value = mock_instance
-        mock_instance.to_string.return_value = "string data"
-
-        self.qdevice_with_ip.write_qdevice_config()
-
-        mock_conf.assert_has_calls([mock.call(), mock.call()])
-        mock_parser.assert_called_once_with("data")
-        mock_instance.remove.assert_called_once_with("quorum.device")
-        mock_instance.add.assert_has_calls([
-            mock.call('quorum', ["device {", "}"]),
-            mock.call('quorum.device', ["net {", "}"])
-        ])
-        mock_instance.set.assert_has_calls([
-            mock.call('quorum.device.votes', '1'),
-            mock.call('quorum.device.model', 'net'),
-            mock.call('quorum.device.net.tls', 'on'),
-            mock.call('quorum.device.net.host', '10.10.10.123'),
-            mock.call('quorum.device.net.port', 5403),
-            mock.call('quorum.device.net.algorithm', 'ffsplit'),
-            mock.call('quorum.device.net.tie_breaker', 'lowest')
-        ])
-        mock_instance.to_string.assert_called_once_with()
-        mock_mksection.assert_has_calls([
-            mock.call('quorum.device', []),
-            mock.call('quorum.device.net', [])
-        ])
-        mock_str2file.assert_called_once_with("string data", "corosync.conf")
-
-    @mock.patch("crmsh.utils.str2file")
-    @mock.patch("crmsh.corosync.Parser")
-    @mock.patch("crmsh.corosync.conf")
-    @mock.patch("crmsh.utils.read_from_file")
-    def test_remove_qdevice_config(self, mock_read_file, mock_conf, mock_parser, mock_str2file):
-        mock_conf.side_effect = ["corosync.conf", "corosync.conf"]
-        mock_read_file.return_value = "data"
-        mock_instance = mock.Mock()
-        mock_parser.return_value = mock_instance
-        mock_instance.to_string.return_value = "string data"
-
-        self.qdevice_with_ip.remove_qdevice_config()
-
-        mock_conf.assert_has_calls([mock.call(), mock.call()])
-        mock_parser.assert_called_once_with("data")
-        mock_instance.remove.assert_called_once_with("quorum.device")
-        mock_instance.to_string.assert_called_once_with()
-        mock_str2file.assert_called_once_with("string data", "corosync.conf")
-
     @mock.patch("crmsh.parallax.parallax_call")
     @mock.patch('crmsh.utils.list_cluster_nodes')
     @mock.patch('os.path.exists')
@@ -855,6 +790,7 @@ Membership information
         mock_status_long.return_value.__exit__ = mock.Mock()
         self.qdevice_with_ip.certificate_process_on_init = mock.Mock()
         self.qdevice_with_ip.adjust_sbd_watchdog_timeout_with_qdevice = mock.Mock()
+        self.qdevice_with_ip.config_qnetd_port = mock.Mock()
         self.qdevice_with_ip.config_qdevice = mock.Mock()
         self.qdevice_with_ip.start_qdevice_service = mock.Mock()
 
@@ -867,20 +803,52 @@ Membership information
         self.qdevice_with_ip.config_qdevice.assert_called_once_with()
         self.qdevice_with_ip.start_qdevice_service.assert_called_once_with()
 
+    @mock.patch('crmsh.utils.check_port_open')
+    @mock.patch('crmsh.qdevice.ServiceManager')
+    def test_config_qnetd_port_no_firewall(self, mock_service, mock_check_port):
+        mock_service_instance = mock.Mock()
+        mock_service.return_value = mock_service_instance
+        mock_service_instance.service_is_active.return_value = False
+
+        self.qdevice_with_ip.config_qnetd_port()
+
+        mock_service_instance.service_is_active.assert_called_once_with("firewalld.service", "10.10.10.123")
+        mock_check_port.assert_not_called()
+
+    @mock.patch('logging.Logger.info')
+    @mock.patch('crmsh.sh.cluster_shell')
+    @mock.patch('crmsh.utils.check_port_open')
+    @mock.patch('crmsh.qdevice.ServiceManager')
+    def test_config_qnetd_port(self, mock_service, mock_check_port, mock_cluster_shell, mock_info):
+        mock_service_instance = mock.Mock()
+        mock_service.return_value = mock_service_instance
+        mock_service_instance.service_is_active.return_value = True
+        mock_check_port.return_value = False
+        mock_cluster_shell_instance = mock.Mock()
+        mock_cluster_shell.return_value = mock_cluster_shell_instance
+        mock_cluster_shell_instance.get_rc_stdout_stderr_without_input = mock.Mock(return_value=(0, None, None))
+        mock_cluster_shell_instance.get_stdout_or_raise_error = mock.Mock(return_value=None)
+        self.qdevice_with_ip.port = 5403
+
+        self.qdevice_with_ip.config_qnetd_port()
+
+        mock_service_instance.service_is_active.assert_called_once_with("firewalld.service", "10.10.10.123")
+        mock_check_port.assert_called_once_with("10.10.10.123", 5403)
+        mock_info.assert_called_once_with("Add port 5403 to firewalld on 10.10.10.123")
+        mock_cluster_shell_instance.get_stdout_or_raise_error.assert_called_once_with("firewall-cmd --reload", "10.10.10.123")
+
     @mock.patch('crmsh.utils.set_property')
     @mock.patch('crmsh.sbd.SBDTimeout.get_stonith_timeout')
-    @mock.patch('crmsh.sbd.SBDManager.update_configuration')
-    @mock.patch('crmsh.sbd.SBDManager.get_sbd_value_from_config')
-    @mock.patch('crmsh.sbd.SBDManager.is_using_diskless_sbd')
-    @mock.patch('crmsh.utils.check_all_nodes_reachable')
-    def test_adjust_sbd_watchdog_timeout_with_qdevice(self, mock_check_reachable, mock_using_diskless_sbd, mock_get_sbd_value, mock_update_config, mock_get_timeout, mock_set_property):
+    @mock.patch('crmsh.sbd.SBDManager.update_sbd_configuration')
+    @mock.patch('crmsh.sbd.SBDUtils.get_sbd_value_from_config')
+    @mock.patch('crmsh.sbd.SBDUtils.is_using_diskless_sbd')
+    def test_adjust_sbd_watchdog_timeout_with_qdevice(self, mock_using_diskless_sbd, mock_get_sbd_value, mock_update_config, mock_get_timeout, mock_set_property):
         mock_using_diskless_sbd.return_value = True
         mock_get_sbd_value.return_value = ""
         mock_get_timeout.return_value = 100
 
         self.qdevice_with_stage_cluster_name.adjust_sbd_watchdog_timeout_with_qdevice()
 
-        mock_check_reachable.assert_called_once_with()
         mock_using_diskless_sbd.assert_called_once_with()
         mock_get_sbd_value.assert_called_once_with("SBD_WATCHDOG_TIMEOUT")
         mock_update_config.assert_called_once_with({"SBD_WATCHDOG_TIMEOUT": str(sbd.SBDTimeout.SBD_WATCHDOG_TIMEOUT_DEFAULT_WITH_QDEVICE)})
@@ -954,29 +922,7 @@ Membership information
         mock_enable_qnetd.assert_called_once_with()
         mock_start_qnetd.assert_called_once_with()
 
-    @mock.patch('crmsh.utils.cluster_run_cmd')
-    @mock.patch('crmsh.bootstrap.update_expected_votes')
-    @mock.patch('crmsh.log.LoggerUtils.status_long')
-    @mock.patch('crmsh.corosync.add_nodelist_from_cmaptool')
-    @mock.patch('crmsh.corosync.is_unicast')
-    @mock.patch('crmsh.qdevice.QDevice.write_qdevice_config')
-    def test_config_qdevice(self, mock_write, mock_is_unicast, mock_add_nodelist, mock_status_long,
-            mock_update_votes, mock_run):
-        mock_is_unicast.return_value = False
-        mock_status_long.return_value.__enter__ = mock.Mock()
-        mock_status_long.return_value.__exit__ = mock.Mock()
-        self.qdevice_with_ip.qdevice_reload_policy = qdevice.QdevicePolicy.QDEVICE_RELOAD
-
-        self.qdevice_with_ip.config_qdevice()
-
-        mock_write.assert_called_once_with()
-        mock_is_unicast.assert_called_once_with()
-        mock_add_nodelist.assert_called_once_with()
-        mock_status_long.assert_called_once_with("Update configuration")
-        mock_update_votes.assert_called_once_with()
-        mock_run.assert_called_once_with("crm corosync reload")
-
-    @mock.patch('crmsh.utils.is_qdevice_configured')
+    @mock.patch('crmsh.corosync.is_qdevice_configured')
     def test_remove_certification_files_on_qnetd_return(self, mock_configured):
         mock_configured.return_value = False
         qdevice.QDevice.remove_certification_files_on_qnetd()
@@ -984,7 +930,7 @@ Membership information
 
     @mock.patch('crmsh.sh.ClusterShell.get_stdout_or_raise_error')
     @mock.patch('crmsh.corosync.get_value')
-    @mock.patch('crmsh.utils.is_qdevice_configured')
+    @mock.patch('crmsh.corosync.is_qdevice_configured')
     def test_remove_certification_files_on_qnetd(self, mock_configured, mock_get_value, mock_run):
         mock_configured.return_value = True
         mock_get_value.side_effect = ["qnetd-node", "cluster1"]
@@ -1000,3 +946,31 @@ Membership information
         mock_run.assert_has_calls([
             mock.call(crt_cmd, "qnetd-node"),
             mock.call(crq_cmd, "qnetd-node")])
+
+    @mock.patch('crmsh.corosync.conf')
+    @mock.patch('crmsh.corosync.ConfParser')
+    def test_remove_qdevice_config(self, mock_parser, mock_conf):
+        mock_parser_inst = mock.Mock()
+        mock_parser.return_value = mock_parser_inst
+        mock_parser_inst.convert2string.return_value = "data"
+        mock_conf.return_value = "corosync.conf"
+        self.qdevice_with_invalid_cmds_relative_path.remove_qdevice_config()
+        mock_parser_inst.remove.assert_called_once_with("quorum.device")
+        mock_parser_inst.save.assert_called_once()
+
+    @mock.patch('crmsh.sh.cluster_shell')
+    @mock.patch('crmsh.bootstrap.sync_file')
+    @mock.patch('crmsh.corosync.configure_two_node')
+    @mock.patch('crmsh.log.LoggerUtils.status_long')
+    @mock.patch('crmsh.qdevice.QDevice.write_qdevice_config')
+    def test_config_qdevice(self, mock_write_config, mock_status_long, mock_config_two_node, mock_sync_file, mock_cluster_shell):
+        mock_cluster_shell_instance = mock.Mock()
+        mock_cluster_shell.return_value = mock_cluster_shell_instance
+        mock_status_long.return_value.__enter__ = mock.Mock()
+        mock_status_long.return_value.__exit__ = mock.Mock()
+        self.qdevice_with_ip.qdevice_reload_policy = qdevice.QdevicePolicy.QDEVICE_RELOAD
+
+        self.qdevice_with_ip.config_qdevice()
+        mock_status_long.assert_called_once_with("Update configuration")
+        mock_config_two_node.assert_called_once_with(qdevice_adding=True)
+        mock_cluster_shell_instance.get_stdout_or_raise_error.assert_called_once_with("corosync-cfgtool -R")
