@@ -13,7 +13,7 @@ class TestCollect(unittest.TestCase):
     def test_get_pcmk_log_no_config(self, mock_isfile, mock_warning):
         mock_isfile.side_effect = [False, False, False]
         res = collect.get_pcmk_log()
-        self.assertEqual(res, "")
+        self.assertIsNone(res)
         mock_isfile.assert_has_calls([
             mock.call(constants.PCMKCONF),
             mock.call("/var/log/pacemaker/pacemaker.log"),
@@ -41,13 +41,18 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
             ])
         mock_read.assert_called_once_with(constants.PCMKCONF)
 
+    @mock.patch('crmsh.report.utils.mark_duplicate_basenames')
     @mock.patch('crmsh.report.utils.dump_logset')
     @mock.patch('os.path.isfile')
     @mock.patch('crmsh.report.collect.get_pcmk_log')
     @mock.patch('crmsh.report.collect.get_corosync_log')
-    def test_collect_ha_logs(self, mock_corosync_log, mock_get_log, mock_isfile, mock_dump):
+    def test_collect_ha_logs(self, mock_corosync_log, mock_get_log, mock_isfile, mock_dump, mock_mark):
         mock_corosync_log.return_value = "/var/log/cluster/corosync.log"
         mock_get_log.return_value = "/var/pacemaker.log"
+        mock_mark.return_value = [
+            (mock_get_log.return_value, False),
+            (mock_corosync_log.return_value, False)
+        ]
         mock_isfile.side_effect = [True, True]
         mock_ctx_inst = mock.Mock(extra_log_list=[])
 
@@ -59,8 +64,8 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
             mock.call(mock_corosync_log.return_value)
             ])
         mock_dump.assert_has_calls([
-            mock.call(mock_ctx_inst, mock_get_log.return_value),
-            mock.call(mock_ctx_inst, mock_corosync_log.return_value)
+            mock.call(mock_ctx_inst, mock_get_log.return_value, create_dir=False),
+            mock.call(mock_ctx_inst, mock_corosync_log.return_value, create_dir=False)
             ])
 
     @mock.patch('logging.Logger.warning')
@@ -69,7 +74,7 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
     def test_get_corosync_log_not_exist(self, mock_conf, mock_exists, mock_warning):
         mock_conf.return_value = "/etc/corosync/corosync.conf"
         mock_exists.return_value = False
-        self.assertEqual(collect.get_corosync_log(), "")
+        self.assertIsNone(collect.get_corosync_log())
 
     @mock.patch('crmsh.corosync.get_value')
     @mock.patch('os.path.exists')
@@ -127,31 +132,6 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
             mock.call(f"Dump jounal log for sbd into {constants.JOURNAL_SBD_F}")
             ])
 
-    @mock.patch('crmsh.report.collect.ShellUtils')
-    def test_dump_D_process_empty(self, mock_run):
-        mock_run_inst = mock.Mock()
-        mock_run.return_value = mock_run_inst
-        mock_run_inst.get_stdout_stderr.return_value = (0, None, None)
-        res = collect.dump_D_process()
-        self.assertEqual(res, "Dump D-state process stack: 0\n")
-
-    @mock.patch('crmsh.report.collect.ShellUtils')
-    def test_dump_D_process(self, mock_run):
-        mock_run_inst = mock.Mock()
-        mock_run.return_value = mock_run_inst
-        mock_run_inst.get_stdout_stderr.side_effect = [
-            (0, "1000", None),
-            (0, "data1", None),
-            (0, "data2", None)
-        ]
-        res = collect.dump_D_process()
-        self.assertEqual(res, "Dump D-state process stack: 1\npid: 1000     comm: data1\ndata2\n\n")
-        mock_run_inst.get_stdout_stderr.assert_has_calls([
-            mock.call("ps aux|awk '$8 ~ /^D/{print $2}'"),
-            mock.call('cat /proc/1000/comm'),
-            mock.call('cat /proc/1000/stack')
-            ])
-
     @mock.patch('logging.Logger.debug')
     @mock.patch('os.path.exists')
     def test_collect_sbd_info_no_config(self, mock_exists, mock_debug):
@@ -187,7 +167,7 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
         mock_open_write = mock.mock_open()
         file_handle = mock_open_write.return_value.__enter__.return_value
         mock_open_file.return_value = mock_open_write.return_value
-        mock_run.return_value = "data"
+        mock_run.side_effect = ["data", "data", "data"]
         mock_ctx_inst = mock.Mock(work_dir="/opt")
 
         collect.collect_sbd_info(mock_ctx_inst)
@@ -199,6 +179,12 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
         file_handle.write.assert_has_calls([
             mock.call("\n\n#=====[ Command ] ==========================#\n"),
             mock.call("# . /etc/sysconfig/sbd;export SBD_DEVICE;sbd dump;sbd list\n"),
+            mock.call("data"),
+            mock.call("\n\n#=====[ Command ] ==========================#\n"),
+            mock.call("# crm sbd configure show\n"),
+            mock.call("data"),
+            mock.call("\n\n#=====[ Command ] ==========================#\n"),
+            mock.call("# crm sbd status\n"),
             mock.call("data")
             ])
         mock_debug.assert_called_once_with(f"Dump SBD config file into {constants.SBD_F}")
@@ -375,74 +361,6 @@ PCMK_logfile=/var/log/pacemaker/pacemaker.log
         ])
         mock_logger.debug.assert_called_with(f'Dump RA trace files into {mock_real_path.return_value}')
 
-    @mock.patch('crmsh.report.collect.ShellUtils')
-    def test_lsof_ocfs2_device(self, mock_run):
-        mock_run_inst = mock.Mock()
-        mock_run.return_value = mock_run_inst
-        mount_data = """
-/dev/vda3 on /home type xfs (rw,relatime,attr2,inode64,logbufs=8,logbsize=32k,noquota)
-tmpfs on /run/user/0 type tmpfs (rw,nosuid,nodev,relatime,size=169544k,nr_inodes=42386,mode=700,inode64)
-/dev/sda7 on /srv/clusterfs type ocfs2 (rw,relatime,heartbeat=non
-        """
-        mock_run_inst.get_stdout_stderr.side_effect = [(0, mount_data, None), (0, "data", None)]
-        res = collect.lsof_ocfs2_device()
-        self.assertEqual(res, "\n\n#=====[ Command ] ==========================#\n# lsof /dev/sda7\ndata")
-        mock_run_inst.get_stdout_stderr.assert_has_calls([
-            mock.call("mount"),
-            mock.call("lsof /dev/sda7")
-        ])
-
-    @mock.patch('crmsh.report.utils.get_cmd_output')
-    @mock.patch('os.path.exists')
-    @mock.patch('shutil.which')
-    def test_ocfs2_commands_output(self, mock_which, mock_exists, mock_run):
-        mock_which.side_effect = [False for i in range(5)] + [True, True]
-        mock_exists.return_value = False
-        mock_run.return_value = "data"
-        res = collect.ocfs2_commands_output()
-        self.assertEqual(res, "\n\n#===== [ Command ] ==========================#\n# mount\ndata")
-
-    @mock.patch('crmsh.report.collect.logger', spec=crmsh.log.DEBUG2Logger)
-    @mock.patch('crmsh.utils.str2file')
-    @mock.patch('crmsh.report.collect.ShellUtils')
-    def test_collect_ocfs2_info_error(self, mock_run, mock_str2file, mock_debug2):
-        mock_run_inst = mock.Mock()
-        mock_run.return_value = mock_run_inst
-        mock_run_inst.get_stdout_stderr.return_value = (1, None, "error")
-        mock_ctx_inst = mock.Mock(work_dir="/opt/workdir")
-        collect.collect_ocfs2_info(mock_ctx_inst)
-        mock_str2file.assert_called_once_with('Failed to run "mounted.ocfs2 -d": error', '/opt/workdir/ocfs2.txt')
-
-    @mock.patch('crmsh.report.collect.logger', spec=crmsh.log.DEBUG2Logger)
-    @mock.patch('crmsh.utils.str2file')
-    @mock.patch('crmsh.report.collect.ShellUtils')
-    def test_collect_ocfs2_info_no_found(self, mock_run, mock_str2file, mock_debug2):
-        mock_run_inst = mock.Mock()
-        mock_run.return_value = mock_run_inst
-        mock_run_inst.get_stdout_stderr.return_value = (0, "data", None)
-        mock_ctx_inst = mock.Mock(work_dir="/opt/workdir")
-        collect.collect_ocfs2_info(mock_ctx_inst)
-        mock_str2file.assert_called_once_with('No ocfs2 partitions found', '/opt/workdir/ocfs2.txt')
-
-    @mock.patch('crmsh.report.utils.real_path')
-    @mock.patch('crmsh.report.collect.ocfs2_commands_output')
-    @mock.patch('crmsh.report.collect.lsof_ocfs2_device')
-    @mock.patch('crmsh.report.collect.dump_D_process')
-    @mock.patch('crmsh.report.collect.logger', spec=crmsh.log.DEBUG2Logger)
-    @mock.patch('crmsh.utils.str2file')
-    @mock.patch('crmsh.report.collect.ShellUtils')
-    def test_collect_ocfs2_info(self, mock_run, mock_str2file, mock_debug2, mock_D, mock_lsof, mock_output, mock_real_path):
-        mock_real_path.return_value = constants.OCFS2_F
-        mock_run_inst = mock.Mock()
-        mock_run.return_value = mock_run_inst
-        mock_run_inst.get_stdout_stderr.return_value = (0, "line1\nline2", None)
-        mock_D.return_value = "data_D\n"
-        mock_lsof.return_value = "data_lsof\n"
-        mock_output.return_value = "data_output\n"
-        mock_ctx_inst = mock.Mock(work_dir="/opt/workdir")
-        collect.collect_ocfs2_info(mock_ctx_inst)
-        mock_str2file.assert_called_once_with('data_D\ndata_lsof\ndata_output\n', '/opt/workdir/ocfs2.txt')
-
     @mock.patch('crmsh.report.utils.real_path')
     @mock.patch('logging.Logger.debug')
     @mock.patch('crmsh.utils.str2file')
@@ -593,21 +511,119 @@ id            0x19041a12
         collect.collect_corosync_blackbox(mock_ctx_inst)
         mock_debug.assert_called_once_with(f"Dump corosync blackbox info into {constants.COROSYNC_RECORDER_F}")
 
-    @mock.patch('logging.Logger.debug')
-    @mock.patch('crmsh.report.utils.real_path')
-    @mock.patch('crmsh.utils.str2file')
-    @mock.patch('crmsh.corosync.query_qnetd_status')
-    @mock.patch('crmsh.corosync.query_qdevice_status')
-    @mock.patch('crmsh.service_manager.ServiceManager')
     @mock.patch('crmsh.corosync.query_quorum_status')
-    def test_collect_qdevice_info(self, mock_quorum, mock_service, mock_qdevice, mock_qnetd, mock_str2file, mock_real_path, mock_debug):
+    @mock.patch('crmsh.report.collect.ServiceManager')
+    def test_collect_qdevice_info_return(self, mock_service, mock_quorum):
         mock_ctx_inst = mock.Mock(work_dir="/opt/workdir")
         mock_service_inst = mock.Mock()
         mock_service.return_value = mock_service_inst
-        mock_service_inst.service_is_active.return_value = True
-        mock_quorum.return_value = "quorum_data"
+        mock_service_inst.service_is_active.return_value = False
+        collect.collect_qdevice_info(mock_ctx_inst)
+        mock_quorum.assert_not_called()
+
+    @mock.patch('logging.Logger.debug')
+    @mock.patch('crmsh.report.utils.real_path')
+    @mock.patch('crmsh.utils.str2file')
+    @mock.patch('os.path.join')
+    @mock.patch('crmsh.corosync.query_qnetd_status')
+    @mock.patch('crmsh.corosync.query_qdevice_status')
+    @mock.patch('crmsh.corosync.query_quorum_status')
+    @mock.patch('crmsh.report.collect.ServiceManager')
+    def test_collect_qdevice_info(self, mock_service, mock_quorum, mock_qdevice, mock_qnetd, mock_join, mock_str2file, mock_real_path, mock_debug):
+        mock_ctx_inst = mock.Mock(work_dir="/opt/workdir")
+        mock_service_inst = mock.Mock()
+        mock_service.return_value = mock_service_inst
+        mock_service_inst.service_is_active.side_effect = [True, True]
         mock_qdevice.return_value = "qdevice_data"
         mock_qnetd.return_value = "qnetd_data"
+        mock_quorum.return_value = "quorum_data"
+        mock_join.return_value = "/opt/workdir/qdevice.txt"
         mock_real_path.return_value = "/opt/workdir/qdevice.txt"
         collect.collect_qdevice_info(mock_ctx_inst)
-        mock_debug.assert_called_once_with(f"Dump quorum/qdevice/qnetd information into {mock_real_path.return_value}")
+        mock_debug.assert_called_once_with('Dump quorum/qdevice/qnetd information into /opt/workdir/qdevice.txt')
+
+    @mock.patch("logging.Logger.error")
+    @mock.patch('crmsh.utils.str2file')
+    @mock.patch("crmsh.report.utils.real_path")
+    @mock.patch("logging.Logger.debug")
+    @mock.patch("crmsh.report.collect.cluster_fs_commands_output")
+    @mock.patch("crmsh.report.collect.lsof_cluster_fs_device")
+    @mock.patch("crmsh.report.collect.dump_D_process")
+    @mock.patch("crmsh.report.collect.ShellUtils")
+    @mock.patch("shutil.which")
+    def test_collect_cluster_fs_info(self, mock_which, mock_run, mock_dump, mock_lsof, mock_cluster, mock_debug, mock_real_path, mock_str2file, mock_error):
+        mock_which.return_value = True
+        mock_run_inst = mock.Mock()
+        mock_run.return_value = mock_run_inst
+        mock_run_inst.get_stdout_stderr.side_effect = [
+            (1, None, "error"),
+            (0, "data", None)
+        ]
+        mock_dump.return_value = "dump_data\n"
+        mock_lsof.return_value = "lsof_data\n"
+        mock_cluster.return_value = "cluster_data"
+        mock_ctx_inst = mock.Mock(work_dir="/opt/work")
+        mock_real_path.side_effect = ["/path/ocfs2.txt", "/path/gfs2.txt"]
+
+        collect.collect_cluster_fs_info(mock_ctx_inst)
+
+        mock_debug.assert_has_calls([
+            mock.call('Dump %s information into %s', 'OCFS2', '/path/ocfs2.txt'),
+            mock.call('Dump %s information into %s', 'GFS2', '/path/gfs2.txt')
+        ])
+        mock_str2file.assert_has_calls([
+            mock.call('Failed to run "mounted.ocfs2 -d": error', '/opt/work/ocfs2.txt'),
+            mock.call('dump_data\nlsof_data\ncluster_data', '/opt/work/gfs2.txt')
+        ])
+
+    @mock.patch('crmsh.report.collect.ShellUtils')
+    def test_dump_D_process_empty(self, mock_run):
+        mock_run_inst = mock.Mock()
+        mock_run.return_value = mock_run_inst
+        mock_run_inst.get_stdout_stderr.return_value = (0, None, None)
+        res = collect.dump_D_process()
+        self.assertEqual(res, "Dump D-state process stack: 0\n")
+
+    @mock.patch('crmsh.report.collect.ShellUtils')
+    def test_dump_D_process(self, mock_run):
+        mock_run_inst = mock.Mock()
+        mock_run.return_value = mock_run_inst
+        mock_run_inst.get_stdout_stderr.side_effect = [
+            (0, "1000", None),
+            (0, "data1", None),
+            (0, "data2", None)
+        ]
+        res = collect.dump_D_process()
+        self.assertEqual(res, "Dump D-state process stack: 1\npid: 1000     comm: data1\ndata2\n\n")
+        mock_run_inst.get_stdout_stderr.assert_has_calls([
+            mock.call("ps aux|awk '$8 ~ /^D/{print $2}'"),
+            mock.call('cat /proc/1000/comm'),
+            mock.call('cat /proc/1000/stack')
+            ])
+
+    @mock.patch('crmsh.report.collect.ShellUtils')
+    def test_lsof_cluster_fs_device(self, mock_run):
+        mock_run_inst = mock.Mock()
+        mock_run.return_value = mock_run_inst
+        mount_data = """
+/dev/vda3 on /home type xfs (rw,relatime,attr2,inode64,logbufs=8,logbsize=32k,noquota)
+tmpfs on /run/user/0 type tmpfs (rw,nosuid,nodev,relatime,size=169544k,nr_inodes=42386,mode=700,inode64)
+/dev/sda7 on /srv/clusterfs type ocfs2 (rw,relatime,heartbeat=non
+        """
+        mock_run_inst.get_stdout_stderr.side_effect = [(0, mount_data, None), (0, "data", None)]
+        res = collect.lsof_cluster_fs_device("OCFS2")
+        self.assertEqual(res, "\n\n#=====[ Command ] ==========================#\n# lsof /dev/sda7\ndata")
+        mock_run_inst.get_stdout_stderr.assert_has_calls([
+            mock.call("mount"),
+            mock.call("lsof /dev/sda7")
+        ])
+
+    @mock.patch('crmsh.report.utils.get_cmd_output')
+    @mock.patch('os.path.exists')
+    @mock.patch('shutil.which')
+    def test_cluster_fs_commands_output(self, mock_which, mock_exists, mock_run):
+        mock_which.side_effect = [False for i in range(5)] + [True, True]
+        mock_exists.return_value = False
+        mock_run.return_value = "data"
+        res = collect.cluster_fs_commands_output("OCFS2")
+        self.assertEqual(res, "\n\n#===== [ Command ] ==========================#\n# mounted.ocfs2 -f\ndata")
