@@ -24,10 +24,10 @@ class TestCrmMonXmlParser(unittest.TestCase):
     <node name="tbw-2" id="1084783312" online="false" standby="false" standby_onfail="false" maintenance="false" pending="false" unclean="false" shutdown="false" expected_up="true" is_dc="false" resources_running="2" type="member"/>
   </nodes>
   <resources>
-    <resource id="ocfs2-dlm" resource_agent="ocf::pacemaker:controld" role="Started" active="true" orphaned="false" blocked="false" managed="true" failed="false" failure_ignored="false" nodes_running_on="1">
+    <resource id="ocfs2-dlm" resource_agent="ocf:pacemaker:controld" role="Started" active="true" orphaned="false" blocked="false" managed="true" failed="false" failure_ignored="false" nodes_running_on="1">
       <node name="tbw-2" id="1084783312" cached="true"/>
     </resource>
-    <resource id="ocfs2-clusterfs" resource_agent="ocf::heartbeat:Filesystem" role="Started" active="true" orphaned="false" blocked="false" managed="true" failed="false" failure_ignored="false" nodes_running_on="1">
+    <resource id="ocfs2-clusterfs" resource_agent="ocf:heartbeat:Filesystem" role="Started" active="true" orphaned="false" blocked="false" managed="true" failed="false" failure_ignored="false" nodes_running_on="1">
       <node name="tbw-2" id="1084783312" cached="true"/>
     </resource>
   </resources>
@@ -41,21 +41,21 @@ class TestCrmMonXmlParser(unittest.TestCase):
         assert self.parser_inst.is_node_online("tbw-2") is False
 
     def test_get_node_list(self):
-        assert self.parser_inst.get_node_list("standby") == ['tbw-1']
-        assert self.parser_inst.get_node_list("online") == ['tbw-2']
+        assert self.parser_inst.get_node_list(standby=True) == ['tbw-1']
+        assert self.parser_inst.get_node_list(online=False) == ['tbw-2']
 
     def test_is_resource_configured(self):
         assert self.parser_inst.is_resource_configured("test") is False
-        assert self.parser_inst.is_resource_configured("ocf::heartbeat:Filesystem") is True
+        assert self.parser_inst.is_resource_configured("ocf:heartbeat:Filesystem") is True
 
-    def test_is_any_resource_running(self):
-        assert self.parser_inst.is_any_resource_running() is True
+    def test_is_non_stonith_resource_running(self):
+        assert self.parser_inst.is_non_stonith_resource_running() is True
 
     def test_is_resource_started(self):
         assert self.parser_inst.is_resource_started("test") is False
         assert self.parser_inst.is_resource_started("ocfs2-clusterfs") is True
-        assert self.parser_inst.is_resource_started("ocf::pacemaker:controld") is True
+        assert self.parser_inst.is_resource_started("ocf:pacemaker:controld") is True
 
     def test_get_resource_id_list_via_type(self):
         assert self.parser_inst.get_resource_id_list_via_type("test") == []
-        assert self.parser_inst.get_resource_id_list_via_type("ocf::pacemaker:controld")[0] == "ocfs2-dlm"
+        assert self.parser_inst.get_resource_id_list_via_type("ocf:pacemaker:controld")[0] == "ocfs2-dlm"
